#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDApplication.py 758 2009-06-05 11:54:30Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#

from EDImportSystem              import PyOs

from EDImportLib                 import EDVerbose
from EDImportLib                 import EDCompiler
from EDImportLib                 import EDString
from EDImportLib                 import EDList
from EDImportLib                 import EDCommandLine
from EDImportLib                 import EDDiskExplorer
from EDImportLib                 import EDTime
from EDImportLib                 import EDDictionary

from EDALApplication             import EDALApplication
from EDConfiguration             import EDConfiguration
from EDMessage                   import EDMessage
from EDUtilsPath                 import EDUtilsPath
from EDUtilsFile                 import EDUtilsFile
from EDFactoryPlugin             import EDFactoryPlugin


class EDApplication( EDALApplication ):

    m_edStringCONFIGURATION_PARAM_LABEL = "--conf"
    m_edStringPLUGIN_PARAM_LABEL = "--execute"
    m_edStringDATASET_PARAM_LABEL = "--inputFile"
    m_edStringOUTPUT_PARAM_LABEL = "--outputFile"
    m_edStringDATASET_BASE_DIRECTORY = "--basedir"
   
    m_edConfiguration = None

    m_edFactoryPlugin = None 
    
    """
    This is the main EDNA application class. This should be sub-classed for any specific application need.
    An EDNA application is able to launch an entry point plugin. It accepts the following parameter:
    --execute : name of the plugin to be executed 
    --inputFile : related plugin data (xml input data file name)
    --outputFile : related plugin result (xml output data file name)
    --conf : configuration file name
    --basedir : where the application working directory should go 
    """
    def __init__( self, _edStringPluginName = None, _strConfigurationFileName = None, _edStringDatasetFileName = None, _edLogFile = None, _edStringBaseDir = None, _edStringWorkingDir = None, _edStringOutputFilePath = None ):
        """
        """

        EDALApplication.__init__( self, "EDApplication", "1.0", _edLogFile )
        
        if( _edStringPluginName == None ):
            self.m_edStringPluginName = self.getCommandLineArgument( EDApplication.m_edStringPLUGIN_PARAM_LABEL )
        else:
            self.m_edStringPluginName = _edStringPluginName
            
        self.m_edStringPlugin = None
            
        # If _strConfigurationFileName is None, this means that it has not been given to the constructor\
        # It has been given by the command line\
        if( _strConfigurationFileName == None ):
            self.m_edStringConfigurationFileName = self.getCommandLineArgument( EDApplication.m_edStringCONFIGURATION_PARAM_LABEL )
        else:
            self.m_edStringConfigurationFileName = _strConfigurationFileName
            
        if( self.m_edStringConfigurationFileName is None and self.m_edStringPluginName is not None ):
            edStringConfigurationHome = EDApplication.getConfigurationHome( self.m_edStringPluginName )
            self.m_edStringConfigurationFileName = EDUtilsPath.mergePath( edStringConfigurationHome, "XSConfiguration_%s.xml" % EDUtilsPath.getEdnaSite())
            
        if( _edStringDatasetFileName == None ):
            self.m_edStringDatasetFileName = self.getCommandLineArgument( EDApplication.m_edStringDATASET_PARAM_LABEL )
        else:
            self.m_edStringDatasetFileName = _edStringDatasetFileName
            
        if( _edStringBaseDir == None ):
            self.m_edStringBaseDir = self.getCommandLineArgument( EDApplication.m_edStringDATASET_BASE_DIRECTORY )
        else:
            self.m_edStringBaseDir = _edStringBaseDir
        
        if( _edStringWorkingDir is not None ):
            self.m_edStringWorkingDir = _edStringWorkingDir
        else:
            self.m_edStringWorkingDir = EDApplication.getApplicationName()
        self.m_edStringFullApplicationWorkingDirectory = None

        self.m_edStringXMLData = None

        self.m_edListErrorMessages = EDList()
        self.m_edListWarningMessages = EDList()
        
        self.m_xsDataOutput = None
        
        self.m_edStringOutputFilePath = None
        if( _edStringOutputFilePath == None ):
            self.m_edStringOutputFilePath = self.getCommandLineArgument( EDApplication.m_edStringOUTPUT_PARAM_LABEL )
        else:
            self.m_edStringOutputFilePath = _edStringOutputFilePath
        

    def getFactoryPlugin():
        EDApplication.synchronizeOn()
        if ( EDApplication.m_edFactoryPlugin is None ):
            EDApplication.m_edFactoryPlugin = EDFactoryPlugin()
        EDApplication.synchronizeOff()
        return EDApplication.m_edFactoryPlugin
    getFactoryPlugin = staticmethod( getFactoryPlugin )
    
    
    def loadPlugin( _edStringPluginName ):
        return EDApplication.getFactoryPlugin().loadPlugin( _edStringPluginName )
    loadPlugin = staticmethod( loadPlugin )

    
    def loadModule( _edStringModuleName ):
        EDApplication.getFactoryPlugin().loadModule( _edStringModuleName )
    loadModule = staticmethod( loadModule )

    
    def setConfiguration( self, _edConfiguration ):
        """
        """
        EDVerbose.DEBUG( "*** EDApplication.setConfiguration" )
        EDApplication.synchronizeOn()
        if ( _edConfiguration == None ):
            EDVerbose.warning( "*** EDApplication.setConfiguration: Configuration is None!" )
        else:               
            EDApplication.m_edConfiguration = _edConfiguration

        EDApplication.synchronizeOff()
        
        
    def getPluginConfiguration( _pluginName ):
        """
        """
        EDVerbose.DEBUG( "*** EDApplication.getPluginConfiguration" )
        EDApplication.synchronizeOn()
        pluginConfiguration = None
        if ( EDApplication.m_edConfiguration != None ):
            pluginConfiguration = EDApplication.m_edConfiguration.getPluginItem( _pluginName )
        else:
            EDVerbose.warning( "*** EDApplication.getPluginConfiguration: No configuration at all!" ) 
        EDApplication.synchronizeOff()
        return pluginConfiguration
    getPluginConfiguration = staticmethod( getPluginConfiguration )
    
    
    def loadConfiguration( self ):
        """
        Loads the configuration file if not already loaded
        """
        if(( EDApplication.m_edConfiguration != None) & (EDApplication.m_edConfiguration.isLoaded() == False )):
            
            EDVerbose.DEBUG( "*** EDApplication.loadConfiguration: Loading Configuration File..." )
            EDApplication.m_edConfiguration.load()
            
            
    def getConfigurationHome( _edStringPluginName ):
        """
        Returns the configuration directory path for a given test module
        """
        edStringModuleLocation = EDApplication.getFactoryPlugin().getModuleLocation( _edStringPluginName )
        edStringConfigurationHome = EDUtilsPath.appendListOfPaths( edStringModuleLocation, [ "..", "..", "..", "conf" ] )
        return edStringConfigurationHome
    getConfigurationHome = staticmethod( getConfigurationHome )

    
    def getDatasetFileName( self ):
        """
        """
        return self.m_edStringDatasetFileName
    
    
    def getBaseDir( self ):
        """
        """
        return self.m_edStringBaseDir
    

    def preProcess( self ):
        """
        Creates the application working directory (log dir)
        Initializes the configuration
        retrieves the plugin xml data to be passed to the plugin
        """

        self.createApplicationWorkingDirectory()
        if ( self.m_edStringConfigurationFileName is not None ):
            EDVerbose.screen( "Loading Configuration file: %s"  % self.m_edStringConfigurationFileName )
            edConfiguration = EDConfiguration( self.m_edStringConfigurationFileName )
            self.setConfiguration( edConfiguration )
            self.loadConfiguration()
            
            EDVerbose.DEBUG( "*** EDApplication.preProcess: Checking... Number of plugins...: " + EDString( EDApplication.m_edConfiguration.getPluginListSize()) )
    
            # specific to EDApplication
            # Children can override preProcessAction for specificities
            self.preProcessAction()
        else:
            EDVerbose.screen( "ERROR! No configuration file found." )

        
    def preProcessAction( self ):
        """
        """
        edDictionary = EDDictionary()
        edDictionary.setValue("${EDNA_HOME}", EDUtilsPath.getEdnaHome() )

        strXMLData = EDUtilsFile.readFileAndParseVariables( self.getDatasetFileName(), edDictionary )
        self.m_edStringXMLData = EDString( strXMLData )
        
        
    def createApplicationWorkingDirectory( self ):
        """
        Created the working directory of the application (<date>-<application name>)
        First tries to retrieve the base dir from --basedir option or related parameter from constructor
        Otherwise tries to retrieve it from EDNA_BASE_DIRECTORY environment variable
        Otherwise put the base dir as the current directory
        """
        EDVerbose.DEBUG( "*** EDApplication.createApplicationWorkingDirectory" )
        edStringBaseDirectory = self.getBaseDir()
        
        if( edStringBaseDirectory is None):
            EDVerbose.DEBUG( "*** EDApplication.createApplicationWorkingDirectory: edna basedir not defined, Trying to get EDNA_BASE_DIRECTORY..." )
            edStringBaseDirectory = PyOs.environ.get("EDNA_BASE_DIRECTORY")
        if ( edStringBaseDirectory is None):
            EDVerbose.DEBUG( "*** EDApplication.createApplicationWorkingDirectory: edna basedir not defined, Using current base directory as working directory." )
            edStringBaseDirectory = EDDiskExplorer.getCwd()
        edStringDateTime = EDTime.getTimeName()
        
        self.m_edStringFullApplicationWorkingDirectory = EDDiskExplorer.mergePath( edStringBaseDirectory, 
                                                                         "%s_%s" % \
                                                                         ( edStringDateTime, \
                                                                           self.m_edStringWorkingDir ) )
        # Check that a folder / file with the same name already exists
        if( EDDiskExplorer.existFolder( self.m_edStringFullApplicationWorkingDirectory ) or \
            EDDiskExplorer.existFile( self.m_edStringFullApplicationWorkingDirectory ) ):
            # It does exist so we have to modify the name of the working directory
            iIndex = 1
            bContinueFlag = True
            while ( bContinueFlag ):
                self.m_edStringFullApplicationWorkingDirectory = EDDiskExplorer.mergePath( edStringBaseDirectory, 
                                                                                        "%s_%s_%d" % \
                                                                                        ( edStringDateTime, \
                                                                                          self.m_edStringWorkingDir, \
                                                                                          iIndex ) )
                if( EDDiskExplorer.existFolder( self.m_edStringFullApplicationWorkingDirectory ) or \
                    EDDiskExplorer.existFile( self.m_edStringFullApplicationWorkingDirectory ) ):
                    iIndex += 1
                else:
                    bContinueFlag = False
        # Make the directory
        EDDiskExplorer.createFolder( self.m_edStringFullApplicationWorkingDirectory )
        
        
    def process( self ):
        """
        Calls the Plugin to be executed
        """          
        
        self.m_edStringPlugin = EDApplication.loadPlugin( self.m_edStringPluginName )
        
        if( self.m_edStringPlugin != None ):

            self.m_edStringPlugin.setBaseDirectory( self.m_edStringFullApplicationWorkingDirectory )
            self.m_edStringPlugin.setBaseName( self.m_edStringPluginName )
            self.m_edStringPlugin.setDataInput( self.m_edStringXMLData )
            
            self.m_edStringPlugin.connectSUCCESS( self.doSuccessActionPlugin ) 
            self.m_edStringPlugin.connectFAILURE( self.doFailureActionPlugin )
            
            EDVerbose.DEBUG( "*** EDApplication.process: Connecting " + self.m_edStringPluginName + " execution..." )
            self.m_edStringPlugin.execute( )
            self.m_edStringPlugin.synchronize()
        else:
            EDVerbose.error( EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ('EDApplication.process', self.m_edStringPluginName ) )
            
            
    def postProcess( self ):
        """
        """
        EDALApplication.postProcess( self )
        self.postProcessAction()
        
        
    def postProcessAction( self ):
        """
        This method can be overloaded by children if needed
        """
        pass
        
        
    def doSuccessActionPlugin( self, _edPlugin ):
        """
        """
        EDVerbose.DEBUG("*** EDApplication.doSuccessActionPlugin" )
        
        # Print the potential Warnings and Errors
        self.m_edListWarningMessages = _edPlugin.getWarningMessages()
        EDVerbose.DEBUG( "*** EDApplication.doSuccessActionPlugin: Plugin " + _edPlugin.getPluginName() + " Successful with : " + EDString( self.m_edListWarningMessages.getNumberObjects() ) + " Warnings: " )
        for warningMessage in self.m_edListWarningMessages:
            EDVerbose.screen( warningMessage )
         
        self.m_edListErrorMessages = _edPlugin.getErrorMessages()   
        EDVerbose.DEBUG( "*** EDApplication.doSuccessActionPlugin: Plugin " + _edPlugin.getPluginName() + " Successful with : " + EDString( self.m_edListErrorMessages.getNumberObjects() ) + " Errors: " )
        for errorMessage in self.m_edListErrorMessages:
            EDVerbose.error( errorMessage )
            
        xsDataOutput = _edPlugin.getDataOutput()
        if ( xsDataOutput is not None and self.m_edStringOutputFilePath is not None ):
            xsDataOutput.outputFile( self.m_edStringOutputFilePath )


    def doFailureActionPlugin( self, _edPlugin ):
        """
        """
        EDVerbose.DEBUG("*** EDApplication.doFailureActionPlugin" )
        
        # Print the potential Warnings and Errors
        self.m_edListWarningMessages = _edPlugin.getWarningMessages()
        EDVerbose.screen( "*** EDApplication.doFailureActionPlugin: Plugin " + _edPlugin.getPluginName() + " Failed with : " + EDString( self.m_edListWarningMessages.getNumberObjects() ) + " Warnings: " )
        for warningMessage in self.m_edListWarningMessages:
            EDVerbose.screen( warningMessage )
        
        self.m_edListErrorMessages = _edPlugin.getErrorMessages()
        EDVerbose.screen( "*** EDApplication.doFailureActionPlugin: Plugin " + _edPlugin.getPluginName() + " Failed with : " + EDString( self.m_edListErrorMessages.getNumberObjects() ) + " Errors: \n" )
        for errorMessage in self.m_edListErrorMessages:
            EDVerbose.screen( errorMessage )
            
            
    def getPlugin( self ):
        """
        """
        return self.m_edStringPlugin
        
        
    def getPluginOutputData( self ):
        """
        """
        return self.m_xsDataOutput
    
    
    def getWarningMessages( self ):
        """
        """
        return self.m_edListWarningMessages
        

    def getErrorMessages( self ):
        """
        """
        return self.m_edListErrorMessages 

    
