#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDConfiguration.py 664 2009-05-06 13:05:34Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib          import EDObject
from EDImportLib          import EDString
from EDImportLib          import EDVerbose
from EDImportLib          import EDDiskExplorer
from EDImportLib          import EDXsdDataBinding


class EDConfiguration( EDObject ):
    """
    This class should be seen as a utilitary class to retrieve param value in a 
    straightforward way. Ideally, subclassing the generated classes from the xml/xsd 
    should be the right way to go (EDConfiguration form XSConfiguration, EDPluginItem
    from XSPluginItem...) but the generated classes a not pure, they are modules that
    contain utilitary methods also (i.e XsdDataBinding uses a method module.unmarshalling
    that is a module method and not a class method)... so subclassing is not possible.
    """
    
    def __init__( self, _edStringXMLFileName, _edStringXSDFileName = None, _bGenerate = False ):
        """
        The Constructor initializes the files (xsd and xml) objects
        The Configuration will be fully constructed after the load method
        """
        # Configuration File
        self.m_edStringXmlFileName = EDString( EDDiskExplorer.absPath( _edStringXMLFileName ) )
        # XSD file
        self.m_edStringXsdFileName = EDString( EDDiskExplorer.absPath( _edStringXSDFileName ) )
        self.m_bGenerate = _bGenerate
        # Object generated automatically with xsd/xml mechanism
        self.m_xsConfiguration = None
        self.m_bLoaded = False
        
        
    def load( self ):
        """
        Loads a configuration File either:
        - in a dynamic way (generation of the python module)
        - or a static way (importing the python Module)
        """
        
        if( self.m_bGenerate == True ):
            self.m_edXsdDataBindingConf = EDXsdDataBinding( self.m_edStringXsdFileName )
            self.m_xsConfiguration = self.m_edXsdDataBindingConf.getObject( self.m_edStringXmlFileName )
        else:
            EDVerbose.DEBUG("*** EDConfiguration.load: importing XSConfiguration module...")
            from XSDataCommon import XSConfiguration
            self.m_xsConfiguration = XSConfiguration.parseFile( self.m_edStringXmlFileName )
            EDVerbose.DEBUG("*** EDConfiguration.load: XSConfiguration module imported")
            
        self.m_bLoaded = True
        
        
    def isLoaded( self ):
        """
        Returns true if the configuration file has been loaded
        """
        return self.m_bLoaded
        
        
    def getXsdFileName( self ):
        """
        Returns the absolute path of the XSD file
        """
        return self.m_edStringXsdFileName
    
    
    def getXmlFileName( self ):
        """
        Returns the absolute path of the XML file
        """
        return self.m_edStringXmlFileName
        
        
    def getXSConfiguration( self ):
        """
        Gets the auto-generated XSConfiguration module
        """
        return self.m_xsConfiguration
    
    
    def getPluginList( self ):
        """
        Returns an XSPluginList object that encapsulates a list of XSPluginItem
        """
        pluginList = self.m_xsConfiguration.getXSPluginList()
        
        return pluginList
    

    def getPluginItem( self, _edStrPluginName):
        """
        Returns a particular XSPluginItem given its name
        """
        xsPluginList = self.m_xsConfiguration.getXSPluginList()
        xsPluginItems = xsPluginList.getXSPluginItem()
        pluginItem = None
        
        for xsPluginItem in xsPluginItems:
            if ( xsPluginItem.getName() == _edStrPluginName ):
               pluginItem = xsPluginItem
               
        return pluginItem
    
    
    def getOptionItem( _xsPluginItem, _pyStrOptionName ):
        """
        Returns a XSOptionItem given an option name
        """
        xsOptionList = _xsPluginItem.getXSOptionList()
        xsOptionItems = xsOptionList.getXSOptionItem()
        optionItem = None
        
        for xsOptionItem in xsOptionItems:
            if ( xsOptionItem.getName() == _pyStrOptionName ):
                optionItem = xsOptionItem
                
        return optionItem
    getOptionItem = staticmethod( getOptionItem )
    
    
    def getParamItem( _xsPluginItem, _pyStrParamName ):
        """
        Returns a XSParamItem given a param name
        """
        xsParamList = _xsPluginItem.getXSParamList()
        xsParamItems = xsParamList.getXSParamItem()
        paramItem = None
        
        for xsParamItem in xsParamItems:
            if ( xsParamItem.getName() == _pyStrParamName ):
                paramItem = xsParamItem
                
        return paramItem
    getParamItem = staticmethod( getParamItem )
    
    
    def getStringParamValue( _xsPluginItem, _pyStrParamName ):
        """
        Returns the parameter value in a string format
        """
        xsParamList = _xsPluginItem.getXSParamList()
        paramValue = None
        
        if ( xsParamList != None ):
            xsParamItems = xsParamList.getXSParamItem()
        
            for xsParamItem in xsParamItems:
                if ( xsParamItem.getName() == _pyStrParamName ):
                    paramValue = xsParamItem.getValue()
                
        return paramValue
    getStringParamValue = staticmethod( getStringParamValue )

    
    def getIntegerParamValue( _xsPluginItem, _pyStrParamName ):
        """
        Returns the parameter value in a integer format
        """
        strParamValue = EDConfiguration.getStringParamValue( _xsPluginItem, _pyStrParamName )
        intParamValue = int( strParamValue )
                
        return intParamValue
    getIntegerParamValue = staticmethod( getIntegerParamValue )
            
    
    def getPluginListSize( self ):
        """
        Returns the size of the XSPluginItem list
        """
        iSize = len( self.getPluginList().getXSPluginItem() )
        return iSize
            
        
        