#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDFactoryPluginTest.py 744 2009-05-29 08:53:14Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal author:    Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author: Marie-Francoise Incardona (incardon@esrf.fr)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#

from EDImportSystem import PyOs 
from EDImportSystem import PySys

from EDFactoryPlugin import EDFactoryPlugin
from EDUtilsPath     import EDUtilsPath

class EDFactoryPluginTest( EDFactoryPlugin ):
    """
    This subclass of EDFactoryPlugin provides a factory for loading test cases for the EDNA
    testing framework. The function of EDFactoryPluginTest is very similar to EDFactoryPlugin,
    the main differences are that only files starting with "EDTest" are considered as test cases
    to be loaded, and that the "src" and "plugin" directories are added to the python path from 
    two possible locations of the test case: 
        
    EDNA application, e.g. mxv1, or plugin collection e.g. mxEcecPlugins
     |
     |-datamodel
     |-src
     |-tests
     |   |-data
     |   |-testsuite
     |   |    |- EDTestSuiteXXX.py
     |
     |-plugins
     |   |-EDPlugin[name of plugin]-v1.0
     |   |   |-plugins
     |   |   |   |-EDPlugin[name of plugin]v10.py
     |   |   |   |-EDPlugin[another name of plugin]v10.py
     |   |   |-tests
     |   |   |   |-data
     |   |   |   |-testsuite
     |   |   |   |    |-EDTestSuiteYYY.py
     
    The "src" directory is searched relatively from both the EDTestSuiteXXX.py and
    EDTestSuiteYYY.py locations, i.e. "../../src" and "../../../../src".
     
    The "plugins" directory is searched only from the EDTestSuiteYYY.py location,
    i.e.  "../../plugins".
    
    The EDFactoryPluginTest also allows tests to be located in $EDNA_HOME/libraries.
    """
    
    def __init__( self ):
        EDFactoryPlugin.__init__( self )
        # Allow tests to be located in $EDNA_HOME/libraries by 
        # not excluding the path to the linraries in the method
        # EDFactoryPlugin.__addPluginLocation.
        # The handling of the paths to exclude should be improved, see bug #306
        self.m_edStringPathToLibraries = None
    
    
    def isPlugin( self, _edStringFileName ):
        """
        For the testing framework, the plugins are test cases and test suites which 
        starts with "EDTest" and ends with ".py".
        """
        bValue = False
        if ( _edStringFileName.startswith( "EDTest" ) and \
             _edStringFileName.endswith( ".py" )  ):
            bValue = True
        return bValue
        
        
    def appendPath( self, _edStringPluginTestLocation ):
        """
        For the tests, both the plugin directory and its corresponding "src" directory must
        be on the python path (see the class documentation for EDFactoryPluginTest).
        """
        EDFactoryPlugin.appendPath( self, _edStringPluginTestLocation )
        edStringSrcDirectory = EDUtilsPath.appendListOfPaths( _edStringPluginTestLocation, [ "..", "..", "..", "..", "src" ] )
        if ( PyOs.path.exists( edStringSrcDirectory ) ):
            if ( not edStringSrcDirectory in PySys.path ):
                PySys.path.append( edStringSrcDirectory )
        edStringSrcDirectory = EDUtilsPath.appendListOfPaths( _edStringPluginTestLocation, [ "..", "..", "src" ] )
        if ( PyOs.path.exists( edStringSrcDirectory ) ):
            if ( not edStringSrcDirectory in PySys.path ):
                PySys.path.append( edStringSrcDirectory )
        edStringPluginDirectory = EDUtilsPath.appendListOfPaths( _edStringPluginTestLocation, [ "..", "..", "plugins" ] )
        if ( PyOs.path.exists( edStringPluginDirectory ) ):
            if ( not edStringPluginDirectory in PySys.path ):
                PySys.path.append( edStringPluginDirectory )
        
    
    
        