#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################



from ALSystem    import ALSystem

"""
Implements the ALSubProcess model for Java-Jython.
"""


if (ALSystem.getInterpreter()=="Jython"):
    """
    Implements the official Jython 2.2 subprocess model.
    """

    from ALImportJython     import *
    from ALObject           import ALObject
    from ALString           import ALString
    from ALVerbose          import ALVerbose
    from ALDiskExplorer     import ALDiskExplorer
    from ALList             import ALList
    from ALSystem           import ALSystem
    from ALTime             import ALTime
    from ALDebug            import ALDebug


    class ALJythonSubProcess( ALObject ):
        """
        Interface for ALSubProcess objects in Jython.
        ref: http://java.sun.com/javase/6/docs/api/java/lang/ProcessBuilder.html#start()
             http://java.sun.com/javase/6/docs/api/java/lang/Process.html
        """
        def __init__( self, _oStrCommandArgs, _oStringFolderName = None ):
            ALObject.__init__( self )
            self.m_ojyProcess = None
            self.m_oalStrCommand = ALString( _oStrCommandArgs )
            # Workaround due the limitation on the java interface to control external processes
            # It will improved with new process interface manager to be developed
            self.m_RETURN_CODE_REFERENCE = -99999.99999
            self.m_oReturnCode  = self.m_RETURN_CODE_REFERENCE
            self.m_obIsFinished = False
            self.m_obIsStarted  = False

            self.m_ojyStdInput    = None
            self.m_ojyStdError    = None
            self.m_ojyStdOutput   = None

            if (_oStringFolderName==None):
                self.m_oStringFolderName = ALSystem.getCurrentFolder()
            else:
                self.m_oStringFolderName = _oStringFolderName
                ALDiskExplorer.createFolder( self.m_oStringFolderName )



        def setWorkingDirectory( self, _oStringFolderName ):
            self.setFolder( _oStringFolderName )


        def setDirectory( self, _oStringFolderName ):
            self.setFolder( _oStringFolderName )


        def setWorkingFolder( self, _oStringFolderName ):
            self.setFolder( _oStringFolderName )


        def setFolder( self, _oStringFolderName ):
            self.synchronizeOn()
            self.m_oStringFolderName = ALDiskExplorer.getAbsolutePath( _oStringFolderName )
            ALDiskExplorer.createFolder( self.m_oStringFolderName )
            self.synchronizeOff()


        def getInput( self ):
            self.synchronizeOn()
            oListStdInput = ALList()
            oListStdInput.add( self.m_oalStrCommand )
            self.synchronizeOff()
            return oListStdInput



        def getOutputError( self ):
            self.synchronizeOn()
            oListStdError = ALList()
            if (self.m_obIsStarted == True and self.m_ojyStdError.ready()):
                ojyStringLine =  self.m_ojyStdError.readLine()
                while ( ojyStringLine != None ):
                    oListStdError.add( ojyStringLine )
                    ojyStringLine =  self.m_ojyStdError.readLine()
            self.synchronizeOff()
            return oListStdError



        def getOutput( self ):
            self.synchronizeOn()
            oListStdOutput = ALList()
            if (self.m_obIsStarted == True and self.m_ojyStdOutput.ready()):
                ojyStringLine =  self.m_ojyStdOutput.readLine()
                while ( ojyStringLine != None ):
                    oListStdOutput.add( ojyStringLine )
                    ojyStringLine =  self.m_ojyStdOutput.readLine()
            self.synchronizeOff()
            return oListStdOutput




        def getReturnCode( self ):
            self.synchronizeOn()
            try:
                # The java can not say if the process is still running
                # The  java creates a stupid exception?????? and after it lost the exitValue()
                if (self.m_ojyProcess!=None):
                    oReturnCode = self.m_ojyProcess.exitValue()
                else:
                    oReturnCode = 0
                self.m_oReturnCode = oReturnCode
            finally:
                ALVerbose.aalibDEBUG( "The Java Process interface crashes when it calls the exitValue() status" )
                ALVerbose.aalibDEBUG( "The Java Process interface will be deleted in the AALib" )
                ALVerbose.aalibDEBUG( "The AALib will implement a complete...version" )
                self.m_oReturnCode = self.m_RETURN_CODE_REFERENCE
                self.m_oReturnCode = 1
            self.synchronizeOff()
            return self.m_oReturnCode


        def __raiseControl( self ):
            oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
            ALDebug.controlDebug( self, oBoolStatus )



        def executeProcess( self, _oalObject = None ):
            ojyFolderName = JyFile( self.m_oStringFolderName )
            self.m_ojyProcess = JyRuntime.getRuntime().exec(self.m_oalStrCommand, None, ojyFolderName  )
            self.m_obIsStarted  = True


            ojyInputReader = JyInputStreamReader( self.m_ojyProcess.getInputStream() )
            self.m_ojyStdOutput = JyBufferedReader( ojyInputReader )

            ojyErrorReader = JyInputStreamReader( self.m_ojyProcess.getErrorStream() )
            self.m_ojyStdError = JyBufferedReader( ojyErrorReader )

            return self.getOutput(), self.getOutputError()


        def getCommunication( self, _oalObject = None ):
            return self.executeProcess( _oalObject )



##############################################################################