#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginMOSFLMIntegrationv10.py 675 2009-05-07 08:42:36Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author:    Karl Levik (karl.levik@diamond.ac.uk)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#

from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDDiskExplorer
 
from EDMessage         import EDMessage
from EDUtilsTable      import EDUtilsTable
from EDPluginMOSFLMv10 import EDPluginMOSFLMv10

from XSDataMOSFLMv10 import XSDataMOSFLMInputIntegration
from XSDataMOSFLMv10 import XSDataMOSFLMOutputIntegration
from XSDataMOSFLMv10 import XSDataMOSFLMBeamPosition
from XSDataMOSFLMv10 import XSDataMOSFLMIntegrationStatistics
from XSDataMOSFLMv10 import XSDataMOSFLMIntegrationStatisticsPerReflectionType
from XSDataMOSFLMv10 import XSDataMOSFLMIntegrationStatisticsPerResolutionBin

from XSDataDnaTables import dna_tables

from XSDataCommon import XSDataAngle
from XSDataCommon import XSDataLength
from XSDataCommon import XSDataInteger
from XSDataCommon import XSDataFloat
from XSDataCommon import XSDataString
from XSDataCommon import XSDataFile


class EDPluginMOSFLMIntegrationv10( EDPluginMOSFLMv10 ):


    def __init__( self ):
        EDPluginMOSFLMv10.__init__( self )
        self.setXSDataInputClass( XSDataMOSFLMInputIntegration )


    def preProcess( self, _edObject = None ):
        EDPluginMOSFLMv10.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMIntegrationv10.preProcess")
        self.generateMOSFLMCommands()


    def postProcess( self, _edObject = None ):
        EDPluginMOSFLMv10.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMIntegrationv10.postProcess")
        xsDataMOSFLMOutputIntegration = self.createDataMOSFLMOutputIntegration()
        if ( xsDataMOSFLMOutputIntegration is not None ):
            self.setDataOutput( xsDataMOSFLMOutputIntegration )


    def configure( self ):
        EDPluginMOSFLMv10.configure( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMIntegrationv10.configure")   
        xsPluginItem = self.getConfiguration()
        if ( xsPluginItem == None ):
            EDVerbose.DEBUG( "*** EDPluginMOSFLMIntegrationv10.configure: xsPluginItem is None")                 


    def checkParameters( self ):
        """
        Checks the mandatory parameters for MOSLFM indexing
        """
        EDPluginMOSFLMv10.checkParameters( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMIntegrationv10.checkParameters")
        self.checkMandatoryParameters( self.getDataInput().getImageEnd(),          "imageEnd" )
        self.checkMandatoryParameters( self.getDataInput().getImageStart(),        "imageStart" )
        self.checkMandatoryParameters( self.getDataInput().getOscillationWidth(),  "oscillationWidth" )
        self.checkMandatoryParameters( self.getDataInput().getRotationAxisStart(), "rotationAxisStart" ) 
        
        
    def generateMOSFLMCommands( self ):
        """
        This method creates a list of MOSFLM integration commands given a valid
        XSDataMOSFLMIntegrationingInput as self.getDataInput()
        """
        EDPluginMOSFLMv10.generateMOSFLMCommands( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMIntegrationv10.generateMOSFLMCommands")

        xsDataMOSFLMInputIntegration = self.getDataInput()

        if ( xsDataMOSFLMInputIntegration is not None):
            
                        
            iImageStart = xsDataMOSFLMInputIntegration.getImageStart().getValue()
            iImageEnd   = xsDataMOSFLMInputIntegration.getImageEnd().getValue()
            fRotationAxisStart = xsDataMOSFLMInputIntegration.getRotationAxisStart().getValue()
            fOscillationWidth = xsDataMOSFLMInputIntegration.getOscillationWidth().getValue()

            self.addListCommandExecution( "HKLOUT %s_process_%d_%d.mtz" % 
                                                       ( self.getScriptBaseName(), iImageStart, iImageEnd ) )
            self.addListCommandExecution( "PROCESS %d TO %d START %f ANGLE %f" %
                                                       ( iImageStart, iImageEnd, fRotationAxisStart, fOscillationWidth ) )
                                                

            self.addListCommandExecution( "BEST ON" )
            self.addListCommandExecution( "GO" )
            self.addListCommandExecution( "BEST OFF" )
            
        EDVerbose.DEBUG( "*** Finished EDPluginMOSFLMIntegrationv10.generateMOSFLMCommands")

            
    def createDataMOSFLMOutputIntegration( self ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMIntegrationv10.createDataMOSFLMOutputIntegration")
        xsDataMOSFLMOutputIntegration = XSDataMOSFLMOutputIntegration()
        # Read bestfile.par, bestfile.hkl and bestfile.dat
        edStringBestfilePar = self.readBestFile( "bestfile.par" )
        if ( edStringBestfilePar is not None ):
            xsDataMOSFLMOutputIntegration.setBestfilePar( XSDataString( edStringBestfilePar ) )
        else:
            xsDataMOSFLMOutputIntegration = None              
          
        if ( xsDataMOSFLMOutputIntegration is not None ):
            edStringBestfileHKL = self.readBestFile( "bestfile.hkl" )
            if ( edStringBestfileHKL is not None ):
                xsDataMOSFLMOutputIntegration.setBestfileHKL( XSDataString( edStringBestfileHKL ) )
            else:
                xsDataMOSFLMOutputIntegration = None              

        if ( xsDataMOSFLMOutputIntegration is not None ):
            edStringBestfileDat = self.readBestFile( "bestfile.dat" )
            if ( edStringBestfileDat is not None ):
                xsDataMOSFLMOutputIntegration.setBestfileDat( XSDataString( edStringBestfileDat ) )
            else:
                xsDataMOSFLMOutputIntegration = None              
          
        if ( xsDataMOSFLMOutputIntegration is not None ):        
            edStringDnaTablesXML = self.readProcessFile( self.getBaseName() + "_dnaTables.xml" )
            xsDataDnaTables = dna_tables.parseString( edStringDnaTablesXML )
            
            xsTableIntegrationResults = EDUtilsTable.getTableFromTables( xsDataDnaTables, "integration_results" )
            xsListFinalResiduals      = EDUtilsTable.getListsFromTable( xsTableIntegrationResults, "final_residuals" )[0]
            edStringRMSSpotDeviation = EDString( EDUtilsTable.getItemFromList( xsListFinalResiduals, "rms" ).getValueOf_() )
            xsDataMOSFLMOutputIntegration.setRMSSpotDeviation( XSDataFloat( edStringRMSSpotDeviation.atof() ) )
            edStringBeamPositionX = EDString( EDUtilsTable.getItemFromList( xsListFinalResiduals, "xcen" ).getValueOf_() )
            edStringBeamPositionY = EDString( EDUtilsTable.getItemFromList( xsListFinalResiduals, "ycen" ).getValueOf_() )
            xsDataMOSFLMBeamPosition = XSDataMOSFLMBeamPosition()
            xsDataMOSFLMBeamPosition.setX( XSDataLength( edStringBeamPositionX.atof() ) )
            xsDataMOSFLMBeamPosition.setY( XSDataLength( edStringBeamPositionY.atof() ) )
            xsDataMOSFLMOutputIntegration.setRefinedBeam( xsDataMOSFLMBeamPosition )
            
            xsDataMOSFLMNewmatMatrix = self.getDataMOSFLMMatrix()
            xsDataMOSFLMOutputIntegration.setRefinedNewmat( xsDataMOSFLMNewmatMatrix )
            
            # New results (described in bug #63)
            
            xsTableIntegrationOutput = EDUtilsTable.getTableFromTables( xsDataDnaTables, "integration_output" )
            xsListOutputFiles = EDUtilsTable.getListsFromTable( xsTableIntegrationOutput, "output_files" )[0]
            
            edStringMTZFilename = EDString( EDUtilsTable.getItemFromList( xsListOutputFiles, "hklout" ).getValueOf_() )
            edStringMTZPath = EDDiskExplorer.mergePath( self.getWorkingDirectory(), edStringMTZFilename )
            xsDataFile = XSDataFile()
            xsDataFile.setPath( XSDataString( edStringMTZPath ))
            xsDataMOSFLMOutputIntegration.setGeneratedMTZFile( xsDataFile )
            
            xsTableSummaryInformation = EDUtilsTable.getTableFromTables( xsDataDnaTables, "summary_information" )
            xsListSummary = EDUtilsTable.getListsFromTable( xsTableSummaryInformation, "summary" )[0]
            xsListSpots = EDUtilsTable.getListsFromTable( xsTableSummaryInformation, "spots" )[0]
            
            edStringDistance = EDString( EDUtilsTable.getItemFromList( xsListSummary, "distance" ).getValueOf_() )
            xsDataMOSFLMOutputIntegration.setRefinedDistance( XSDataLength( edStringDistance.atof() ) )
            
            edStringYScale = EDString( EDUtilsTable.getItemFromList( xsListSummary, "yscale" ).getValueOf_() )
            xsDataMOSFLMOutputIntegration.setRefinedYScale( XSDataFloat( edStringYScale.atof() ) )
            
            edStringOverallIOverSigma = EDString( EDUtilsTable.getItemFromList( xsListSummary, "isigall" ).getValueOf_() )
            xsDataMOSFLMOutputIntegration.setOverallIOverSigma( XSDataFloat( edStringOverallIOverSigma.atof() ) )
            
            edStringHighestResolutionIOverSigma = EDString( EDUtilsTable.getItemFromList( xsListSummary, "isigout" ).getValueOf_() )
            xsDataMOSFLMOutputIntegration.setHighestResolutionIOverSigma( XSDataFloat( edStringHighestResolutionIOverSigma.atof() ) )
            
            edStringNumberOfBadReflections = EDString( EDUtilsTable.getItemFromList( xsListSpots, "bad_spots" ).getValueOf_() )
            xsDataMOSFLMOutputIntegration.setNumberOfBadReflections( XSDataInteger( edStringNumberOfBadReflections.atoi() ) )

            edStringNumberOfFullyRecordedReflections = EDString( EDUtilsTable.getItemFromList( xsListSpots, "full" ).getValueOf_() )
            xsDataMOSFLMOutputIntegration.setNumberOfFullyRecordedReflections( XSDataInteger( edStringNumberOfFullyRecordedReflections.atoi() ) )

            edStringNumberOfNegativeReflections = EDString( EDUtilsTable.getItemFromList( xsListSpots, "negative" ).getValueOf_() )
            xsDataMOSFLMOutputIntegration.setNumberOfNegativeReflections( XSDataInteger( edStringNumberOfNegativeReflections.atoi() ) )
            
            edStringNumberOfOverlappedReflections = EDString( EDUtilsTable.getItemFromList( xsListSpots, "overlap" ).getValueOf_() )
            xsDataMOSFLMOutputIntegration.setNumberOfOverlappedReflections( XSDataInteger( edStringNumberOfOverlappedReflections.atoi() ) )

            edStringNumberOfPartialReflections = EDString( EDUtilsTable.getItemFromList( xsListSpots, "partial" ).getValueOf_() )
            xsDataMOSFLMOutputIntegration.setNumberOfPartialReflections( XSDataInteger( edStringNumberOfPartialReflections.atoi() ) )

            # Fill in the statistics
            
            xsDataMOSFLMOutputIntegration.setOverallStatistics( self.getMOSFLMIntegrationStatisticsPerResolutionBin( xsDataDnaTables, "bin_0" ) )
            
            iIndex = 1            
            bContinue = True
            fMinResolution = None
            while ( bContinue == True ):
                edStringBin = "bin_%d" % iIndex
                # I don't know how many intensity bins there are. Since the EDUtilsTable.getListsFromTable
                # method crashes if the list is not present, I had to wrap the calls to this method in
                # try - except:
                try:
                    xsDataMOSFLMIntegrationStatisticsPerResolutionBin = self.getMOSFLMIntegrationStatisticsPerResolutionBin( xsDataDnaTables, edStringBin, fMinResolution )
                    fMinResolution =  xsDataMOSFLMIntegrationStatisticsPerResolutionBin.getMaxResolution().getValue()
                    xsDataMOSFLMOutputIntegration.addStatisticsPerResolutionBin( xsDataMOSFLMIntegrationStatisticsPerResolutionBin )
                    iIndex += 1
                except:
                    bContinue = False

        return xsDataMOSFLMOutputIntegration
    
    
    def readBestFile( self, _edStringBestfileName ):
        edStringBestfile = None
        try:
            edStringBestfile = self.readProcessFile( _edStringBestfileName )
        except:    
            edStringError = self.readProcessErrorLogFile()
            if( (edStringError is not None) and ( edStringError != EDString.empty() ) ):
                errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginMOSFLMIntegrationv10.readBestFile', 'EDPluginMOSFLMIntegrationv10', edStringError )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
            else:
                errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginMOSFLMIntegrationv10.readBestFile", \
                                                                   "Cannot read MOSFLM " + _edStringBestfileName + " file" )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
        return edStringBestfile
    
    
    def getMOSFLMIntegrationStatisticsPerResolutionBin( self, _xsDataDnaTables, _edStringListName, _fMinResolution = None ):
        """
        This method creates an XSDataMOSFLMIntegrationStatisticsPerResolutionBin object given an XSDataDnaTables object
        and a list name, e.g. "bin_1". If _fMinResolution is provided it is used for setting the minimum resolution,
        otherwise the min and max resolution are set to the same value provided by the xsDataList object.
        """
        xsDataTableProfileFittedFull    = EDUtilsTable.getTableFromTables( _xsDataDnaTables, "profile_fitted_full" )
        xsDataTableProfileFittedPartial = EDUtilsTable.getTableFromTables( _xsDataDnaTables, "profile_fitted_partial" )
        xsDataTableSummationFull        = EDUtilsTable.getTableFromTables( _xsDataDnaTables, "summation_full" )
        xsDataTableSummationPartial     = EDUtilsTable.getTableFromTables( _xsDataDnaTables, "summation_partial" )

        xsListProfileFittedFull    = EDUtilsTable.getListsFromTable( xsDataTableProfileFittedFull, _edStringListName )[0]
        xsListProfileFittedPartial = EDUtilsTable.getListsFromTable( xsDataTableProfileFittedPartial, _edStringListName )[0]
        xsListSummationFull        = EDUtilsTable.getListsFromTable( xsDataTableSummationFull, _edStringListName )[0]
        xsListSummationPartial     = EDUtilsTable.getListsFromTable( xsDataTableSummationPartial, _edStringListName )[0]
        
        xsDataMOSFLMIntegrationDataProfileFittedFull     = self.getMOSFLMIntegrationStatistics( xsListProfileFittedFull )
        xsDataMOSFLMIntegrationDataProfileFittedPartials = self.getMOSFLMIntegrationStatistics( xsListProfileFittedPartial )
        xsDataMOSFLMIntegrationSummationFull             = self.getMOSFLMIntegrationStatistics( xsListSummationFull )
        xsDataMOSFLMIntegrationSummationPartial          = self.getMOSFLMIntegrationStatistics( xsListSummationPartial )

        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeProfileFitted = XSDataMOSFLMIntegrationStatisticsPerReflectionType()
        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeProfileFitted.setFullyRecorded( xsDataMOSFLMIntegrationDataProfileFittedFull )
        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeProfileFitted.setPartials( xsDataMOSFLMIntegrationDataProfileFittedPartials )
        
        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeSummation = XSDataMOSFLMIntegrationStatisticsPerReflectionType()
        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeSummation.setFullyRecorded( xsDataMOSFLMIntegrationSummationFull )
        xsDataMOSFLMIntegrationStatisticsPerReflectionTypeSummation.setPartials( xsDataMOSFLMIntegrationSummationPartial )
        
        xsDataMOSFLMIntegrationStatisticsPerResolutionBin = XSDataMOSFLMIntegrationStatisticsPerResolutionBin()
        xsDataMOSFLMIntegrationStatisticsPerResolutionBin.setProfileFitted( xsDataMOSFLMIntegrationStatisticsPerReflectionTypeProfileFitted )
        xsDataMOSFLMIntegrationStatisticsPerResolutionBin.setSummation( xsDataMOSFLMIntegrationStatisticsPerReflectionTypeSummation )

        edStringMaxResolution = EDString( EDUtilsTable.getItemFromList( xsListProfileFittedFull, "resolution" ).getValueOf_() )
        if ( not edStringMaxResolution.equals( "overall" ) ):
            xsDataMOSFLMIntegrationStatisticsPerResolutionBin.setMaxResolution( XSDataFloat( edStringMaxResolution.atof() ) )
            if ( _fMinResolution is not None ):
                xsDataMOSFLMIntegrationStatisticsPerResolutionBin.setMinResolution( XSDataFloat( _fMinResolution ) )
            else:
                xsDataMOSFLMIntegrationStatisticsPerResolutionBin.setMinResolution( XSDataFloat( edStringMaxResolution.atof() ) )

        return xsDataMOSFLMIntegrationStatisticsPerResolutionBin
    
    

    def getMOSFLMIntegrationStatistics( self, _xsDataList ):
        """
        This method creates a new XSDataMOSFLMIntegrationStatistics object and populates it with
        values (averaged intensity, sigma, average I over sigma and number of reflections) 
        from the _xsDataList object.
        """
        xsDataMOSFLMIntegrationStatistics = XSDataMOSFLMIntegrationStatistics()

        edStringAverageIntensity = EDString( EDUtilsTable.getItemFromList( _xsDataList, "i" ).getValueOf_() )
        xsDataMOSFLMIntegrationStatistics.setAverageIntensity( XSDataFloat( edStringAverageIntensity.atof() ) )

        edStringAverageSigma = EDString( EDUtilsTable.getItemFromList( _xsDataList, "sig" ).getValueOf_() )
        xsDataMOSFLMIntegrationStatistics.setAverageSigma( XSDataFloat( edStringAverageSigma.atof() ) )

        edStringAverageIOverSigma = EDString( EDUtilsTable.getItemFromList( _xsDataList, "i_sig" ).getValueOf_() )
        xsDataMOSFLMIntegrationStatistics.setAverageIOverSigma( XSDataFloat( edStringAverageIOverSigma.atof() ) )

        edStringNumberOfReflections = EDString( EDUtilsTable.getItemFromList( _xsDataList, "count" ).getValueOf_() )
        xsDataMOSFLMIntegrationStatistics.setNumberOfReflections( XSDataInteger( edStringNumberOfReflections.atoi() ) )

        return xsDataMOSFLMIntegrationStatistics
            
    
    def generateExecutiveSummary( self, _edPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDPluginMOSFLMv10.generateExecutiveSummary( self, _edPlugin )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMIntegrationv10.createDataMOSFLMOutputIndexing")
        xsDataMOSFLMInputIntegration  = self.getDataInput()
        xsDataMOSFLMOutputIntegration = self.getDataOutput()
        if ( xsDataMOSFLMOutputIntegration is not None):
            self.addExecutiveSummaryLine( "Execution of MOSFLM integration successful." )
            self.addExecutiveSummaryLine( "Image directory     : %s" %  xsDataMOSFLMInputIntegration.getDirectory().getValue() )
            self.addExecutiveSummaryLine( "Image template      : %s" %  xsDataMOSFLMInputIntegration.getTemplate().getValue() )
            self.addExecutiveSummaryLine( "Image start         : %4d" %  xsDataMOSFLMInputIntegration.getImageStart().getValue() )
            self.addExecutiveSummaryLine( "Image end           : %4d" %  xsDataMOSFLMInputIntegration.getImageEnd().getValue() )
            fRotationAxisStart = xsDataMOSFLMInputIntegration.getRotationAxisStart().getValue()
            fOscillationWidth  = xsDataMOSFLMInputIntegration.getOscillationWidth().getValue()
            self.addExecutiveSummaryLine( "Rotation axis start : %4.1f [degrees]" % fRotationAxisStart )
            self.addExecutiveSummaryLine( "Rotation axis end   : %4.1f [degrees]" % ( fRotationAxisStart + fOscillationWidth ) )
            self.addExecutiveSummaryLine( "" )
            iNumberOfFullyRecordedReflections =  xsDataMOSFLMOutputIntegration.getNumberOfFullyRecordedReflections().getValue()
            self.addExecutiveSummaryLine( "Number of fully recorded reflections          : %5d" % iNumberOfFullyRecordedReflections )
            self.addExecutiveSummaryLine( "Number of partials                            : %5d" % xsDataMOSFLMOutputIntegration.getNumberOfPartialReflections().getValue() )
            self.addExecutiveSummaryLine( "Number of overlapped reflections              : %5d" % xsDataMOSFLMOutputIntegration.getNumberOfOverlappedReflections().getValue() )
            self.addExecutiveSummaryLine( "Number of reflections with negative intensity : %5d" % xsDataMOSFLMOutputIntegration.getNumberOfNegativeReflections().getValue() )
            self.addExecutiveSummaryLine( "Number of bad reflections                     : %5d" % xsDataMOSFLMOutputIntegration.getNumberOfBadReflections().getValue() )
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "RMS spot deviation                            : %5.3f [mm]" % xsDataMOSFLMOutputIntegration.getRMSSpotDeviation().getValue() )
            self.addExecutiveSummaryLine( "Average I/sigma overall                       : %5.1f" % xsDataMOSFLMOutputIntegration.getOverallIOverSigma().getValue() )
            self.addExecutiveSummaryLine( "Average I/sigma at highest resolution         : %5.1f" % xsDataMOSFLMOutputIntegration.getHighestResolutionIOverSigma().getValue() )
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "Analysis as a function of resolution:" )
            edStringResolution                                  = "Res (Ang) "
    
            edStringNumberProfileFittedFullyRecorded            = "Number    "
            edStringAverageIntensityProfileFittedFullyRecorded  = " <I>      "
            edStringAverageIOverSigmaProfileFittedFullyRecorded = " <I/sigma>"
    
            edStringNumberProfileFittedPartials            = "Number    "
            edStringAverageIntensityProfileFittedPartials  = " <I>      "
            edStringAverageIOverSigmaProfileFittedPartials = " <I/sigma>"
              
            for xsDataMOSFLMIntegrationStatisticsPerResolutionBin in xsDataMOSFLMOutputIntegration.getStatisticsPerResolutionBin():
                fMaxResolution = xsDataMOSFLMIntegrationStatisticsPerResolutionBin.getMaxResolution().getValue()
                edStringResolution += "%7.2f" % fMaxResolution
                xsDataMOSFLMIntegrationStatisticsProfileFitted             = xsDataMOSFLMIntegrationStatisticsPerResolutionBin.getProfileFitted()
                
                xsDataMOSFLMIntegrationStatisticsProfileFittedFullyRecorded = xsDataMOSFLMIntegrationStatisticsProfileFitted.getFullyRecorded()
                edStringNumberProfileFittedFullyRecorded            += "%7d" % xsDataMOSFLMIntegrationStatisticsProfileFittedFullyRecorded.getNumberOfReflections().getValue()
                edStringAverageIntensityProfileFittedFullyRecorded  += "%7.0f" % xsDataMOSFLMIntegrationStatisticsProfileFittedFullyRecorded.getAverageIntensity().getValue()
                edStringAverageIOverSigmaProfileFittedFullyRecorded += "%7.1f" % xsDataMOSFLMIntegrationStatisticsProfileFittedFullyRecorded.getAverageIOverSigma().getValue()
                
                xsDataMOSFLMIntegrationStatisticsProfileFittedPartials      = xsDataMOSFLMIntegrationStatisticsProfileFitted.getPartials()
                edStringNumberProfileFittedPartials            += "%7d" % xsDataMOSFLMIntegrationStatisticsProfileFittedPartials.getNumberOfReflections().getValue()
                edStringAverageIntensityProfileFittedPartials  += "%7.0f" % xsDataMOSFLMIntegrationStatisticsProfileFittedPartials.getAverageIntensity().getValue()
                edStringAverageIOverSigmaProfileFittedPartials += "%7.1f" % xsDataMOSFLMIntegrationStatisticsProfileFittedPartials.getAverageIOverSigma().getValue()
                        
            edStringResolution += " Overall"
            self.addExecutiveSummaryLine( edStringResolution )
            
            edStringNumberProfileFittedFullyRecorded            += "%8d"   % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getFullyRecorded().getNumberOfReflections().getValue()
            edStringAverageIntensityProfileFittedFullyRecorded  += "%8.0f" % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getFullyRecorded().getAverageIntensity().getValue()
            edStringAverageIOverSigmaProfileFittedFullyRecorded += "%8.1f" % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getFullyRecorded().getAverageIOverSigma().getValue()
    
            edStringNumberProfileFittedPartials            += "%8d"   % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getPartials().getNumberOfReflections().getValue()
            edStringAverageIntensityProfileFittedPartials  += "%8.0f" % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getPartials().getAverageIntensity().getValue()
            edStringAverageIOverSigmaProfileFittedPartials += "%8.1f" % xsDataMOSFLMOutputIntegration.getOverallStatistics().getProfileFitted().getPartials().getAverageIOverSigma().getValue()
            
            if ( iNumberOfFullyRecordedReflections > 0 ):
                self.addExecutiveSummaryLine( "Profile fitted fully recorded:" )
                self.addExecutiveSummaryLine( edStringNumberProfileFittedFullyRecorded )
                self.addExecutiveSummaryLine( edStringAverageIntensityProfileFittedFullyRecorded )
                self.addExecutiveSummaryLine( edStringAverageIOverSigmaProfileFittedFullyRecorded )
            
            self.addExecutiveSummaryLine( "Profile fitted partials:" )
            self.addExecutiveSummaryLine( edStringNumberProfileFittedPartials )
            self.addExecutiveSummaryLine( edStringAverageIntensityProfileFittedPartials )
            #self.addExecutiveSummaryLine( edStringAverageSigmaProfileFittedPartials )
            self.addExecutiveSummaryLine( edStringAverageIOverSigmaProfileFittedPartials )
                    
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "Path to MTZ file : %s" % xsDataMOSFLMOutputIntegration.getGeneratedMTZFile().getPath().getValue() )