#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginMOSFLMv10.py 724 2009-05-20 11:54:26Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author:    Karl Levik (karl.levik@diamond.ac.uk)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDList
from EDImportLib import EDDiskExplorer

from EDMessage                 import EDMessage
from EDPluginExecProcessScript import EDPluginExecProcessScript

from XSDataCommon import XSDataMatrixDouble
from XSDataCommon import XSDataAngle
from XSDataCommon import XSDataLength

from XSDataMOSFLMv10 import XSDataCell
from XSDataMOSFLMv10 import XSDataMOSFLMNewmat
from XSDataMOSFLMv10 import XSDataMOSFLMMissettingsAngles


class EDPluginMOSFLMv10( EDPluginExecProcessScript ):

    def __init__( self ):
        EDPluginExecProcessScript.__init__( self )
        
        self.addCompatibleVersion( EDString( "Mosflm version 7.0.5 for Image plate and CCD data 10th February 2009" ) )
        self.addCompatibleVersion( EDString( "Mosflm version 7.0.5 for Image plate and CCD data 13th May 2009" ) ) 

        self.m_edStringMOSFLMNewmatFileName = None
        self.m_edStringMOSFLMMatrixFileName = None
        

    def process( self, _edObject = None ):
        EDPluginExecProcessScript.process( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv10.process")
        # It should not be possible to execute this abstract plugin
        if ( self.getPluginName().equals( "EDPluginMOSFLMv10" ) ):
             raise ExectuteAbstractPluginError

    
    def configure( self ):
        EDPluginExecProcessScript.configure( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv10.configure")
        self.setRequireCCP4( True )
        self.setScriptCommandline( " DNA " + self.getScriptBaseName() + "_dnaTables.xml" )
#        if ( self.m_edStringMOSFLMNewmatFileName == None ):
#            self.m_edStringMOSFLMNewmatFileName = EDString( self.m_edStringScriptBaseName + ".mat" ) 


    def checkParameters( self ):
        """
        Checks the mandatory parameters for all MOSLFM plugins
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv10.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                 "Data Input is None" )
        self.checkMandatoryParameters( self.getDataInput().getBeam(),       "beamPosition" )
        self.checkMandatoryParameters( self.getDataInput().getDetector(),   "detector" )
        self.checkMandatoryParameters( self.getDataInput().getDirectory(),  "directory" )
        self.checkMandatoryParameters( self.getDataInput().getDistance(),   "distance" )
        self.checkMandatoryParameters( self.getDataInput().getTemplate(),   "template" )
        self.checkMandatoryParameters( self.getDataInput().getWavelength(), "wavelength" )


    def generateMOSFLMCommands( self ):
        """
        This method creates a list of MOSFLM indexing commands given a valid
        XSDataMOSFLMInput as self.getDataInput()
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv10.generateMOSFLMCommands")
        xsDataMOSFLMInput = self.getDataInput()

        if ( xsDataMOSFLMInput is not None):
                        
            xsDataFloatWavelength = xsDataMOSFLMInput.getWavelength()
            if ( xsDataFloatWavelength is not None ): 
                self.addListCommandExecution( "WAVELENGTH " + EDString( xsDataFloatWavelength.getValue() ) )

            xsDataLengthDistance = xsDataMOSFLMInput.getDistance() 
            if ( xsDataLengthDistance is not None ):
                self.addListCommandExecution( "DISTANCE " + EDString( xsDataLengthDistance.getValue() ) )

            xsDataMOSFLMBeamPosition = xsDataMOSFLMInput.getBeam()
            if ( xsDataMOSFLMBeamPosition is not None ):
                edStringBeamX = EDString( xsDataMOSFLMBeamPosition.getX().getValue() )
                edStringBeamY = EDString( xsDataMOSFLMBeamPosition.getY().getValue() )
                self.addListCommandExecution( "BEAM " + edStringBeamX + " " + edStringBeamY )

            xsDataMOSFLMDetector = xsDataMOSFLMInput.getDetector() 
            if ( xsDataMOSFLMDetector is not None ):
                edStringDetectorType = xsDataMOSFLMDetector.getType()
                if ( edStringDetectorType is not None ):
                    self.addListCommandExecution( "DETECTOR " + edStringDetectorType.getValue() )
                    
            xsDataStringDirectory = xsDataMOSFLMInput.getDirectory()
            if ( xsDataStringDirectory is not None ):
                self.addListCommandExecution( "DIRECTORY " + xsDataStringDirectory.getValue() )

            xsDataStringTemplate = xsDataMOSFLMInput.getTemplate()
            if ( xsDataStringTemplate is not None ):
                self.addListCommandExecution( "TEMPLATE " + xsDataStringTemplate.getValue() )
            
            xsDataStringSymmetry = xsDataMOSFLMInput.getSymmetry()
            if ( xsDataStringSymmetry is not None ):
                self.addListCommandExecution( "SYMMETRY " + xsDataStringSymmetry.getValue() )
            edStringNewmatFileName = self.getNewmatFileName()

            xsDataNewmatMatrix = xsDataMOSFLMInput.getMatrix()
            if ( xsDataNewmatMatrix is not None ):
                self.writeDataMOSFLMNewmat( xsDataNewmatMatrix, self.getMatrixFileName() )
                self.addListCommandExecution( "MATRIX " + self.getMatrixFileName() )
            
            xsDataFloatMosaicity = xsDataMOSFLMInput.getMosaicity()
            if ( xsDataFloatMosaicity is not None ):
                self.addListCommandExecution( "MOSAIC " + EDString( xsDataFloatMosaicity.getValue() ) )


    def getNewmatFileName( self ):
        if ( self.m_edStringMOSFLMNewmatFileName is None ):
            self.m_edStringMOSFLMNewmatFileName = EDString( self.getScriptBaseName() + "_newmat.mat" )
        return EDString( self.m_edStringMOSFLMNewmatFileName )

    
    def setNewmatFileName( self, _ostrMOSFLMNewmatFileName ):
        self.synchronizeOn()
        self.m_edStringMOSFLMNewmatFileName = EDString( _ostrMOSFLMNewmatFileName )
        self.synchronizeOff()
        

    def getMatrixFileName( self ):
        if ( self.m_edStringMOSFLMMatrixFileName is None ):
            self.m_edStringMOSFLMMatrixFileName = EDString( self.getScriptBaseName() + "_matrix.mat" )
        return EDString( self.m_edStringMOSFLMMatrixFileName )

    
    def setMatrixFileName( self, _edStringMOSFLMMatrixFileName ):
        self.synchronizeOn()
        self.m_edStringMOSFLMMatrixFileName = EDString( _edStringMOSFLMMatrixFileName )
        self.synchronizeOff()
    
    
    def splitStringIntoListOfFloats( self, _edStringInput ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv10.splitStringIntoListOfFloats")
        edListFloats = EDList()
        edListString = EDString( _edStringInput ).split()
        for pyStringElement in edListString:
            edStringElement = EDString( pyStringElement )
            if ( edStringElement.equals( "" ) == False ):
                edListFloats.add( EDString( edStringElement ).atof() )
        return edListFloats
    
    
    def getDataMOSFLMMatrix( self, _edStringMatrixFileName = None  ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv10.getDataMOSFLMMatrix")
        edStringMatrixFileName = None
        if ( _edStringMatrixFileName is None ):
            edStringMatrixFileName = self.getMatrixFileName()
        else:
            edStringMatrixFileName = _edStringMatrixFileName
        xsDataMOSFLMNewmatMatrix = self.getDataMOSFLMNewmat( edStringMatrixFileName )
        return xsDataMOSFLMNewmatMatrix
        
    
    def getDataMOSFLMNewmat( self, _edStringNewmatFileName = None ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv10.getDataMOSFLMNewmat")
        xsDataMOSFLMNewmat = None
        edStringNewmatFileName = None
        edListOfListOfFloat = EDList()
        if ( _edStringNewmatFileName == None ):
            edStringNewmatFileName = self.getNewmatFileName()
        else:
            edStringNewmatFileName = _edStringNewmatFileName
        edStringNewmat = None    
        try:
            edStringNewmat = self.readProcessFile( edStringNewmatFileName )
        except:
            edStringError = self.readProcessErrorLogFile()
            if( (edStringError is not None) and ( edStringError != EDString.empty() ) ):
                errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginMOSFLMv10.getDataMOSFLMNewmat', self.getClassName(), edStringError )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
            else:
                errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginMOSFLMv10.getDataMOSFLMNewmat", \
                                                                   "Cannot read MOSFLM NEWMAT file : " + edStringNewmatFileName )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage ) 
        #print edStringNewmat
        if ( edStringNewmat is not None ):
            edListLine = edStringNewmat.split("\n")
            # Convert into list of lists of float
            for edStringLine in edListLine:
                edListOfListOfFloat.add( self.splitStringIntoListOfFloats( edStringLine ) )
            # Fill in the data
            xsDataMOSFLMNewmat = XSDataMOSFLMNewmat()

            XSDataMatrixDoubleA = XSDataMatrixDouble()
            XSDataMatrixDoubleA.setM11( edListOfListOfFloat[0][0] )            
            XSDataMatrixDoubleA.setM12( edListOfListOfFloat[0][1] )
            XSDataMatrixDoubleA.setM13( edListOfListOfFloat[0][2] )
            XSDataMatrixDoubleA.setM21( edListOfListOfFloat[1][0] )
            XSDataMatrixDoubleA.setM22( edListOfListOfFloat[1][1] )
            XSDataMatrixDoubleA.setM23( edListOfListOfFloat[1][2] )
            XSDataMatrixDoubleA.setM31( edListOfListOfFloat[2][0] )
            XSDataMatrixDoubleA.setM32( edListOfListOfFloat[2][1] )
            XSDataMatrixDoubleA.setM33( edListOfListOfFloat[2][2] )
            xsDataMOSFLMNewmat.setAMatrix( XSDataMatrixDoubleA )
            
            xsDataMOSFLMMissettingsAngles = XSDataMOSFLMMissettingsAngles()
            xsDataMOSFLMMissettingsAngles.setPhix( XSDataAngle( edListOfListOfFloat[3][0] ) )
            xsDataMOSFLMMissettingsAngles.setPhiy( XSDataAngle( edListOfListOfFloat[3][1] ) )
            xsDataMOSFLMMissettingsAngles.setPhiz( XSDataAngle( edListOfListOfFloat[3][2] ) )
            xsDataMOSFLMNewmat.setMissettingAngles( xsDataMOSFLMMissettingsAngles )

            XSDataMatrixDoubleU = XSDataMatrixDouble()
            XSDataMatrixDoubleU.setM11( edListOfListOfFloat[4][0] )
            XSDataMatrixDoubleU.setM12( edListOfListOfFloat[4][1] )
            XSDataMatrixDoubleU.setM13( edListOfListOfFloat[4][2] )
            XSDataMatrixDoubleU.setM21( edListOfListOfFloat[5][0] )
            XSDataMatrixDoubleU.setM22( edListOfListOfFloat[5][1] )
            XSDataMatrixDoubleU.setM23( edListOfListOfFloat[5][2] )
            XSDataMatrixDoubleU.setM31( edListOfListOfFloat[6][0] )
            XSDataMatrixDoubleU.setM32( edListOfListOfFloat[6][1] )
            XSDataMatrixDoubleU.setM33( edListOfListOfFloat[6][2] )
            xsDataMOSFLMNewmat.setUMatrix( XSDataMatrixDoubleU )
            
            xsDataCellRefined = XSDataCell()
            xsDataCellRefined.setLength_a(    XSDataLength( edListOfListOfFloat[7][0] ) )
            xsDataCellRefined.setLength_b(    XSDataLength( edListOfListOfFloat[7][1] ) )
            xsDataCellRefined.setLength_c(    XSDataLength( edListOfListOfFloat[7][2] ) )
            xsDataCellRefined.setAngle_alpha( XSDataAngle(  edListOfListOfFloat[7][3] ) )
            xsDataCellRefined.setAngle_beta(  XSDataAngle(  edListOfListOfFloat[7][4] ) )
            xsDataCellRefined.setAngle_gamma( XSDataAngle(  edListOfListOfFloat[7][5] ) )
            xsDataMOSFLMNewmat.setRefinedCell( xsDataCellRefined )
            
        return xsDataMOSFLMNewmat
    
    
    
    def writeDataMOSFLMNewmat( self, _xsDataMOSFLNNewmat, _edStringNewmatFileName = None ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv10.writeDataMOSFLMNewmat")
        edStringNewmatFileName = None
        if ( _edStringNewmatFileName is None ):
            edStringNewmatFileName = self.getNewmatFileName()
        else:
            edStringNewmatFileName = _edStringNewmatFileName
        XSDataMatrixDoubleA = _xsDataMOSFLNNewmat.getAMatrix()
        edStringNewmat  = " %11.8f %11.8f %11.8f\n" % ( XSDataMatrixDoubleA.getM11(), XSDataMatrixDoubleA.getM12(), XSDataMatrixDoubleA.getM13() )
        edStringNewmat += " %11.8f %11.8f %11.8f\n" % ( XSDataMatrixDoubleA.getM21(), XSDataMatrixDoubleA.getM22(), XSDataMatrixDoubleA.getM23() )
        edStringNewmat += " %11.8f %11.8f %11.8f\n" % ( XSDataMatrixDoubleA.getM31(), XSDataMatrixDoubleA.getM32(), XSDataMatrixDoubleA.getM33() )
        xsDataMOSFLMMissettingAngles = _xsDataMOSFLNNewmat.getMissettingAngles()
        edStringNewmat += " %11.3f %11.3f %11.3f\n" % ( xsDataMOSFLMMissettingAngles.getPhix().getValue(),
                                                         xsDataMOSFLMMissettingAngles.getPhiy().getValue(),
                                                         xsDataMOSFLMMissettingAngles.getPhiz().getValue())
        XSDataMatrixDoubleU = _xsDataMOSFLNNewmat.getUMatrix()
        edStringNewmat += " %11.7f %11.7f %11.7f\n" % ( XSDataMatrixDoubleU.getM11(), XSDataMatrixDoubleU.getM12(), XSDataMatrixDoubleU.getM13() )
        edStringNewmat += " %11.7f %11.7f %11.7f\n" % ( XSDataMatrixDoubleU.getM21(), XSDataMatrixDoubleU.getM22(), XSDataMatrixDoubleU.getM23() )
        edStringNewmat += " %11.7f %11.7f %11.7f\n" % ( XSDataMatrixDoubleU.getM31(), XSDataMatrixDoubleU.getM32(), XSDataMatrixDoubleU.getM33() )

        xsDataCellRefined = _xsDataMOSFLNNewmat.getRefinedCell()
        edStringNewmat += " %11.4f %11.4f %11.4f" %   ( xsDataCellRefined.getLength_a().getValue(),
                                                         xsDataCellRefined.getLength_b().getValue(),
                                                         xsDataCellRefined.getLength_c().getValue())
        edStringNewmat += " %11.4f %11.4f %11.4f\n" % ( xsDataCellRefined.getAngle_alpha().getValue(),
                                                         xsDataCellRefined.getAngle_beta().getValue(),
                                                         xsDataCellRefined.getAngle_gamma().getValue())
        edStringNewmat += " %11.3f %11.3f %11.3f\n" % ( xsDataMOSFLMMissettingAngles.getPhix().getValue(),
                                                         xsDataMOSFLMMissettingAngles.getPhiy().getValue(),
                                                         xsDataMOSFLMMissettingAngles.getPhiz().getValue()) 
        
        self.writeProcessFile( edStringNewmatFileName, edStringNewmat )
        
        
        
    def generateExecutiveSummary( self, _edPlugin ):
        """
        Generates a summary of the execution of the plugin.
        This method is common to all MOSFLM plugins.
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMv10.generateExecutiveSummary" )
        if ( self.getStringVersion() is not None ):
            self.addExecutiveSummaryLine( self.getStringVersion() )
