#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginLabelitv10.py 675 2009-05-07 08:42:36Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author:    Karl Levik (karl.levik@diamond.ac.uk)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#

from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDList

from EDConfiguration           import EDConfiguration
from EDMessage                 import EDMessage
from EDPluginExecProcessScript import EDPluginExecProcessScript

from XSDataCommon import XSDataLength
from XSDataCommon import XSDataAngle
from XSDataCommon import XSDataBoolean
from XSDataCommon import XSDataInteger
from XSDataCommon import XSDataFloat
from XSDataCommon import XSDataString
from XSDataCommon import XSDataMatrixDouble
from XSDataCommon import XSPluginItem

from XSDataLabelitv10 import XSDataCell
from XSDataLabelitv10 import XSDataInputLabelit
from XSDataLabelitv10 import XSDataResultLabelit
from XSDataLabelitv10 import XSDataLabelitScreenSolution
from XSDataLabelitv10 import XSDataLabelitScreenOutput
from XSDataLabelitv10 import XSDataLabelitMosflmScriptsOutput

class EDPluginLabelitv10( EDPluginExecProcessScript ):
    """
    This plugin runs the labelit.screen command for indexing reference images.
    If the labelit.sceen command is successful, the labelit.mosflm_scripts command is run
    for generating the orientation matrices (in MOSFLM format).
    
    The plugin requires that the path to the Labelit "setpath.sh" script is configured
    using the configuration keyword "pathToLabelitSetpathScript".
    """
    
    def __init__( self ):
        EDPluginExecProcessScript.__init__( self )
        self.setXSDataInputClass( XSDataInputLabelit )
        self.m_edStringCONF_PATH_TO_LABELIT_SETPATH_SCRIPT = EDString( "pathToLabelitSetpathScript" )        
        self.m_edStringPathToLabelitSetpathScript = None 
        
        
    def configure( self ):
        EDPluginExecProcessScript.configure( self )
        EDVerbose.DEBUG( "*** EDPluginControlLabelitv10.configure" )
        xsPluginItem = self.getConfiguration()
        if ( xsPluginItem == None ):
            EDVerbose.warning( "EDPluginControlLabelitv10.configure: No Labelit plugin item defined." )
            xsPluginItem = XSPluginItem()
        edStringPathToLabelitSetpathScript = EDConfiguration.getStringParamValue( xsPluginItem, self.m_edStringCONF_PATH_TO_LABELIT_SETPATH_SCRIPT )
        if( edStringPathToLabelitSetpathScript == None ):
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ( 'EDPluginControlLabelitv10.configure', self.getClassName(), "Configuration parameter missing: "+self.m_edStringCONF_PATH_TO_LABELIT_SETPATH_SCRIPT ) 
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage             
        else:
            self.setPathToLabelitSetpathScript( edStringPathToLabelitSetpathScript )


    def setPathToLabelitSetpathScript( self, _edStringPathToLabelitSetpathScript ):
        self.m_edStringPathToLabelitSetpathScript = EDString( _edStringPathToLabelitSetpathScript )
        
        
    def getPathToLabelitSetpathScript( self ):
        return self.m_edStringPathToLabelitSetpathScript
        
    
    def preProcess( self, _edObject = None ):
        """
        Sets up the Labelit command line
        """
        EDPluginExecProcessScript.preProcess( self, _edObject )
        EDVerbose.DEBUG( "*** EDPluginLabelitv10.preProcess..." )
        
        self.setScriptExecutable( "labelit.screen" )
        
        self.initaliseLabelitCommandLine()
        
        self.addListCommandPreExecution( "export PYTHONPATH=\"\" " )
        self.addListCommandPreExecution( ". %s" % self.getPathToLabelitSetpathScript() )
        
        self.addListCommandPostExecution( "[ -f \"LABELIT_possible\" ] && labelit.mosflm_scripts" )


    def postProcess( self, _edObject = None  ):
        """
        Parses the labelit.screen log file and the generated MOSFLM script
        """
        EDPluginExecProcessScript.postProcess( self, _edObject )
        EDVerbose.DEBUG( "*** EDPluginLabelitv10.postProcess..." )
        edStringLabelitLog = self.readProcessLogFile()
        if ( edStringLabelitLog is None ):
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ( "EDPluginLabelitv10.postProcess", self.getClassName(), "Could not read the Labelit log file" )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
        else:
            xsDataLabelitScreenOutput = self.parseLabelitScreenOutput( edStringLabelitLog )
            xsDataIntegerSelectedSolutionNumber = xsDataLabelitScreenOutput.getSelectedSolutionNumber()
            if ( xsDataIntegerSelectedSolutionNumber is None ):
                errorMessage = EDMessage.ERROR_EXECUTION_03 % ( "EDPluginLabelitv10.postProcess", self.getClassName(), "No selected solution" )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
            else:
                edStringLabelitMosflmScriptsOutput = self.readProcessFile( self.generateMosflmScriptName( xsDataIntegerSelectedSolutionNumber.getValue() ) ) 
                xsDataLabelitMosflmScriptsOutput = self.parseMosflmScriptsOutput( edStringLabelitMosflmScriptsOutput )
                xsDataResultLabelit = XSDataResultLabelit()
                xsDataResultLabelit.setLabelitScreenOutput( xsDataLabelitScreenOutput )
                xsDataResultLabelit.setLabelitMosflmScriptsOutput( xsDataLabelitMosflmScriptsOutput )
                self.setDataOutput( xsDataResultLabelit )
        
        
    def initaliseLabelitCommandLine( self ):
        """
        Initialises the Labelit.screen command line
        """
        EDVerbose.DEBUG( "*** EDPluginLabelitv10.initaliseLabelitCommandLine" )
        
        edStringCommandLabelit = EDString( "--index_only" )
        xsDataInputLabelit = self.getDataInput()
        xsDataImageList = xsDataInputLabelit.getImage()
        for xsDataImage in xsDataImageList:
            edStringCommandLabelit = edStringCommandLabelit + " " + xsDataImage.getPath().getValue()     
        self.setScriptCommandline( edStringCommandLabelit )


    def parseLabelitScreenOutput( self, _edStringLabelitLogText ):
        """
        This method parses the labelit.screen log and populates the relevant
        parts of the XSDataLabelitScreenOutput object which is then returned.
        """
        EDVerbose.DEBUG( "*** EDPluginLabelitv10.parseLabelitLogText" )
        xsDataLabelitScreenOutput = None
         
        iIndex = 0
        edListLogLines = EDList( _edStringLabelitLogText.split( '\n' ) )
        bFoundLabelitIndexingResults = False
        bContinue = True 
        
        while ( bContinue ) :
            if ( EDString.find( edListLogLines[ iIndex ], 'LABELIT Indexing results:') != -1 ) :
                bFoundLabelitIndexingResults = True
                bContinue = False
            else:
                if ( iIndex < ( len( edListLogLines ) - 2 ) ):
                    iIndex += 1
                else:
                    bContinue = False
                    
        if ( bFoundLabelitIndexingResults == False ):
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ( "EDPluginLabelitv10.parseLabelitLogText", self.getClassName(), "Labelit log message: %s" % _edStringLabelitLogText)
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
        else:
            # We found some indexing results!
            xsDataLabelitScreenOutput = XSDataLabelitScreenOutput()

            iIndex +=1
            edStringBeamDistanceMosaicity = edListLogLines[ iIndex ]
            edStringBeamDistanceMosaicity = edStringBeamDistanceMosaicity.replace( ',', '')
            edStringBeamDistanceMosaicity = edStringBeamDistanceMosaicity.replace( 'mm', ' ')
            edStringBeamDistanceMosaicity = edStringBeamDistanceMosaicity.replace( '=', ' ')
            edListStringBeamDistanceMosaicity = EDList( edStringBeamDistanceMosaicity.split() )

            fBeamX = EDString( edListStringBeamDistanceMosaicity[3] ).atof()
            fBeamY = EDString( edListStringBeamDistanceMosaicity[5] ).atof()
            xsDataLabelitScreenOutput.setBeamCentreX( XSDataLength( fBeamX ) )
            xsDataLabelitScreenOutput.setBeamCentreY( XSDataLength( fBeamY ) )
            fDistance = EDString( edListStringBeamDistanceMosaicity[7] ).atof()
            xsDataLabelitScreenOutput.setDistance( XSDataLength( fDistance ) )

            fMosaicity = EDString( edListStringBeamDistanceMosaicity[11] ).atof()
            xsDataLabelitScreenOutput.setMosaicity( XSDataAngle( fMosaicity ) )

            iIndex += 3
            bContinue = True
            bFoundSelectedSolution = False
            while ( bContinue ):
                xsDataLabelitScreenSolution = XSDataLabelitScreenSolution()
                edStringLabelitSolution = edListLogLines[ iIndex ]
                edListStringLabelitSolution = EDList( edStringLabelitSolution.split() )
                
                if ( EDString( edListStringLabelitSolution[0] ).equals( ":)" ) ):
                    xsDataLabelitScreenSolution.setHappy( XSDataBoolean( True ) )
                else:
                    xsDataLabelitScreenSolution.setHappy( XSDataBoolean( False ) )
                
                iSolutionNumber = EDString( edListStringLabelitSolution[1] ).atoi()
                xsDataLabelitScreenSolution.setSolutionNumber( XSDataInteger( iSolutionNumber ) )
                
                fMetricFitValue = EDString( edListStringLabelitSolution[2] ).atof()
                xsDataLabelitScreenSolution.setMetricFitValue( XSDataFloat( fMetricFitValue ) )
                
                edStringMetricFitCode = EDString( edListStringLabelitSolution[3] )
                xsDataLabelitScreenSolution.setMetricFitCode( XSDataString( edStringMetricFitCode ) )

                fRMSD = EDString( edListStringLabelitSolution[4] ).atof()
                xsDataLabelitScreenSolution.setRmsd( XSDataLength( fRMSD ) )
                
                iNumberOfSpots = EDString( edListStringLabelitSolution[5] ).atoi()
                xsDataLabelitScreenSolution.setNumberOfSpots( XSDataInteger( iNumberOfSpots ) )                
                
                xsDataLabelitScreenSolution.setCrystalSystem( XSDataString( edListStringLabelitSolution[6] ) )
                xsDataLabelitScreenSolution.setBravaisLattice( XSDataString( edListStringLabelitSolution[7] ) )
                
                xsDataCell = XSDataCell()
                xsDataCell.setLength_a(    XSDataLength( EDString( edListStringLabelitSolution[ 8] ).atof() ) )
                xsDataCell.setLength_b(    XSDataLength( EDString( edListStringLabelitSolution[ 9] ).atof() ) )
                xsDataCell.setLength_c(    XSDataLength( EDString( edListStringLabelitSolution[10] ).atof() ) )
                xsDataCell.setAngle_alpha( XSDataAngle(  EDString( edListStringLabelitSolution[11] ).atof() ) )
                xsDataCell.setAngle_beta(  XSDataAngle(  EDString( edListStringLabelitSolution[12] ).atof() ) )
                xsDataCell.setAngle_gamma( XSDataAngle(  EDString( edListStringLabelitSolution[13] ).atof() ) )
                xsDataLabelitScreenSolution.setUnitCell( xsDataCell )
                
                iVolume = EDString( edListStringLabelitSolution[14] ).atoi()
                xsDataLabelitScreenSolution.setVolume( XSDataInteger( iVolume ) )
                
                # Find the selected solution
                if ( bFoundSelectedSolution == False ):
                    if ( xsDataLabelitScreenSolution.getHappy().getValue() == True ):
                        xsDataLabelitScreenOutput.setSelectedSolutionNumber( XSDataInteger( iSolutionNumber ) )
                        bFoundSelectedSolution = True
                
                
                xsDataLabelitScreenOutput.addLabelitScreenSolution( xsDataLabelitScreenSolution )
                if ( iIndex < ( len( edListLogLines ) - 2 ) ):
                    iIndex += 1
                else:
                    bContinue = False

            
        return xsDataLabelitScreenOutput


    def parseMosflmScriptsOutput( self, _edStringMOSFLMScript ):
        """
        This method parses the MOSFLM script generated by Labelit and populates
        the A- and U-matrices in the XSDataLabelitMosflmScriptsOutput object.
        """
        EDVerbose.DEBUG( "*** EDPluginLabelitv10.parseMOSFLMMatrices" )
        xsDataLabelitMosflmScriptsOutput = XSDataLabelitMosflmScriptsOutput()
        edListMOSFLMScriptLines = EDList( _edStringMOSFLMScript.split( "\n" ) )

        XSDataMatrixDoubleA = XSDataMatrixDouble()
        edListMOSFLMScriptLine2 = edListMOSFLMScriptLines[2].split()
        edListMOSFLMScriptLine3 = edListMOSFLMScriptLines[3].split()
        edListMOSFLMScriptLine4 = edListMOSFLMScriptLines[4].split()
        XSDataMatrixDoubleA.setM11( EDString( edListMOSFLMScriptLine2[0] ).atof() )            
        XSDataMatrixDoubleA.setM12( EDString( edListMOSFLMScriptLine2[1] ).atof() )
        XSDataMatrixDoubleA.setM13( EDString( edListMOSFLMScriptLine2[2] ).atof() )
        XSDataMatrixDoubleA.setM21( EDString( edListMOSFLMScriptLine3[0] ).atof() )
        XSDataMatrixDoubleA.setM22( EDString( edListMOSFLMScriptLine3[1] ).atof() )
        XSDataMatrixDoubleA.setM23( EDString( edListMOSFLMScriptLine3[2] ).atof() )
        XSDataMatrixDoubleA.setM31( EDString( edListMOSFLMScriptLine4[0] ).atof() )
        XSDataMatrixDoubleA.setM32( EDString( edListMOSFLMScriptLine4[1] ).atof() )
        XSDataMatrixDoubleA.setM33( EDString( edListMOSFLMScriptLine4[2] ).atof() )
        xsDataLabelitMosflmScriptsOutput.setAMatrix( XSDataMatrixDoubleA )
        
        XSDataMatrixDoubleU = XSDataMatrixDouble()
        edListMOSFLMScriptLine6 = edListMOSFLMScriptLines[6].split()
        edListMOSFLMScriptLine7 = edListMOSFLMScriptLines[7].split()
        edListMOSFLMScriptLine8 = edListMOSFLMScriptLines[8].split()
        XSDataMatrixDoubleU.setM11( EDString( edListMOSFLMScriptLine6[0] ).atof() )            
        XSDataMatrixDoubleU.setM12( EDString( edListMOSFLMScriptLine6[1] ).atof() )
        XSDataMatrixDoubleU.setM13( EDString( edListMOSFLMScriptLine6[2] ).atof() )
        XSDataMatrixDoubleU.setM21( EDString( edListMOSFLMScriptLine7[0] ).atof() )
        XSDataMatrixDoubleU.setM22( EDString( edListMOSFLMScriptLine7[1] ).atof() )
        XSDataMatrixDoubleU.setM23( EDString( edListMOSFLMScriptLine7[2] ).atof() )
        XSDataMatrixDoubleU.setM31( EDString( edListMOSFLMScriptLine8[0] ).atof() )
        XSDataMatrixDoubleU.setM32( EDString( edListMOSFLMScriptLine8[1] ).atof() )
        XSDataMatrixDoubleU.setM33( EDString( edListMOSFLMScriptLine8[2] ).atof() )
        xsDataLabelitMosflmScriptsOutput.setUMatrix( XSDataMatrixDoubleU )
        
        return xsDataLabelitMosflmScriptsOutput
        

    def generateMosflmScriptName( self, _iSelectedSolutionNumber ):
        """
        This method generates the name of the MOSFLM script given a solution number.
        """
        EDVerbose.DEBUG( "*** EDPluginLabelitv10.generateMOSFLMScriptName" )
        return EDString( "integration%02d.csh" % _iSelectedSolutionNumber )
        

    def generateExecutiveSummary( self, _edPlugin ):
        """
        Generates a summary of the execution of the Labelit plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginLabelitv10.generateExecutiveSummary")
        xsDataInputLabelit  = self.getDataInput()
        xsDataResultLabelit = self.getDataOutput()
        xsDataLabelitScreenOutput = xsDataResultLabelit.getLabelitScreenOutput()
        if ( self.getErrorMessages().getNumberObjects() == 0 ):
            self.addExecutiveSummaryLine( "Execution of Labelit indexing successful." )
        iIndex = 1
        for xsDataImage in xsDataInputLabelit.getImage():
             self.addExecutiveSummaryLine( "Image %d                : %s" % (iIndex, xsDataImage.getPath().getValue() ) )
             iIndex += 1
        self.addExecutiveSummaryLine( "" )
        self.addExecutiveSummaryLine( "Labelit Indexing results:" )
        self.addExecutiveSummaryLine( "Beam centre             : x=%5.1f mm, y=%5.1f mm" % \
                                      ( xsDataLabelitScreenOutput.getBeamCentreX().getValue(), \
                                        xsDataLabelitScreenOutput.getBeamCentreY().getValue() ) )
        self.addExecutiveSummaryLine( "Distance                : %5.1f mm" % xsDataLabelitScreenOutput.getDistance().getValue() )
        self.addExecutiveSummaryLine( "Mosaicity               : %5.2f degrees" % xsDataLabelitScreenOutput.getMosaicity().getValue() )
        self.addExecutiveSummaryLine( "" )
        self.addExecutiveSummaryLine( "Solution  Metric fit  rmsd  #spots  crystal_system   unit_cell                                  volume" )
        for xsDataLabelitScreenSolution in xsDataLabelitScreenOutput.getLabelitScreenSolution():
            edStringSmiley = None
            if ( xsDataLabelitScreenSolution.getHappy().getValue() == True ):
                edStringSmiley = ":)"
            else: 
                edStringSmiley = ";("
            self.addExecutiveSummaryLine( "%2s %3d %10.4f %2s %5.3f %6d %13s %2s  %6.2f %6.2f %6.2f %6.2f %6.2f %6.2f %8d" % ( \
                                                           edStringSmiley, \
                                                           xsDataLabelitScreenSolution.getSolutionNumber().getValue(), \
                                                           xsDataLabelitScreenSolution.getMetricFitValue().getValue(), \
                                                           xsDataLabelitScreenSolution.getMetricFitCode().getValue(), \
                                                           xsDataLabelitScreenSolution.getRmsd().getValue(), \
                                                           xsDataLabelitScreenSolution.getNumberOfSpots().getValue(), \
                                                           xsDataLabelitScreenSolution.getCrystalSystem().getValue(), \
                                                           xsDataLabelitScreenSolution.getBravaisLattice().getValue(), \
                                                           xsDataLabelitScreenSolution.getUnitCell().getLength_a().getValue(), \
                                                           xsDataLabelitScreenSolution.getUnitCell().getLength_b().getValue(), \
                                                           xsDataLabelitScreenSolution.getUnitCell().getLength_c().getValue(), \
                                                           xsDataLabelitScreenSolution.getUnitCell().getAngle_alpha().getValue(), \
                                                           xsDataLabelitScreenSolution.getUnitCell().getAngle_beta().getValue(), \
                                                           xsDataLabelitScreenSolution.getUnitCell().getAngle_gamma().getValue(), \
                                                           xsDataLabelitScreenSolution.getVolume().getValue(), \
                                                           ) )
        







