#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginRaddosev10.py 753 2009-06-04 09:54:13Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#

from EDImportLib                  import EDVerbose
from EDImportLib                  import EDString
from EDImportLib                  import EDList
from EDImportLib                  import EDDict
from EDImportLib                  import EDDiskExplorer

from EDMessage                    import EDMessage
from EDUtilsFile                  import EDUtilsFile
from EDPluginExecProcessScript    import EDPluginExecProcessScript

from XSDataCommon                 import XSDataAbsorbedDoseRate
from XSDataCommon                 import XSDataFloat
from XSDataRaddosev10             import XSDataRaddoseInput
from XSDataRaddosev10             import XSDataRaddoseOutput



class EDPluginRaddosev10( EDPluginExecProcessScript ):
    """
    """
    
    # Depending on RADDOSE version, Absorbed keyword in the log file is different
    # in Raddose 1: (actually no version stamped for Raddose coming with ccp4 4.2) keyword is "Dose in Grays"
    # in Raddose 2: keyword is "Total absorbed dose (Gy)"
    m_edListABSORBED_DOSE = EDList( [ EDString( "Dose in Grays" ), EDString("Total absorbed dose (Gy)" ) ] )
    m_edStringSOLVENT = "Solvent Content (%)"
    m_iMIN_SOLVENT_PERCENTAGE = 25
    m_iMAX_SOLVENT_PERCENTAGE = 75
    m_fHENDERSON_LIMIT = 2e7

    def __init__( self ):
        """
        """
        EDPluginExecProcessScript.__init__( self )
        self.m_edStringCommandBeamSize = None
        self.m_edStringCommandBeamFlux = None
        self.m_edStringCommandBeamWavelength = None
        
        self.m_edStringCommandCrystalSize = None
        self.m_edStringCommandCrystalCell = None
        self.m_edStringCommandCrystalNRES = None
        self.m_edStringCommandCrystalNMON = None
        self.m_edStringCommandCrystalNDNA = None
        self.m_edStringCommandCrystalNRNA = None
        self.m_edStringCommandCrystalPATM = None
        self.m_edStringCommandCrystalSATM = None
        
        self.m_edStringCommandExposureTime = None
        self.m_edStringCommandImages = None
        
        self.m_edListCommandsRaddose = EDList()
        
        self.m_edDictResults = EDDict()
        
        # ugly workaround while waiting for RADDOSE XML output file
        for edStringAbsorbedDoseKeyword in EDPluginRaddosev10.m_edListABSORBED_DOSE:
            self.m_edDictResults.add( edStringAbsorbedDoseKeyword, None )
            
        self.m_edDictResults.add( EDPluginRaddosev10.m_edStringSOLVENT, None )
        
        self.m_fSolvent = None
        self.m_fTimeToReachHendersonLimit = None
        
        self.setXSDataInputClass( XSDataRaddoseInput )


    def getCommandCrystalSize( self ):
        """
        """
        return self.m_edStringCommandCrystalSize
    
    
    def getCommandCrystalCell( self ):
        """
        """
        return self.m_edStringCommandCrystalCell
    
    
    def getCommandCrystalNRES( self ):
        """
        """
        return self.m_edStringCommandCrystalNRES
    
    
    def getCommandCrystalNMON( self ):
        """
        """
        return self.m_edStringCommandCrystalNMON
    
    
    def getCommandCrystalNDNA( self ):
        """
        """
        return self.m_edStringCommandCrystalNDNA
    
    
    def getCommandCrystalNRNA( self ):
        """
        """
        return self.m_edStringCommandCrystalNRNA
          
        
    def getCommandCrystalPATM( self ):
        """
        """
        return self.m_edStringCommandCrystalPATM
    
    
    def getCommandCrystalSATM( self ):
        """
        """
        return self.m_edStringCommandCrystalSATM
    
    
    def getCommandBeamSize( self ):
        """
        """
        return self.m_edStringCommandBeamSize
    
    
    def getCommandBeamWavelength( self ):
        """
        """
        return self.m_edStringCommandBeamWavelength


    def getCommandBeamFlux( self ):
        """
        """
        return self.m_edStringCommandBeamFlux
    
    
    def getCommandBeamWavelength( self ):
        """
        """
        return self.m_edStringCommandBeamWavelength
    
    
    def getCommandExposureTime( self ):
        """
        """
        return self.m_edStringCommandExposureTime
    
    
    def getCommandImages( self ):
        """
        """
        return self.m_edStringCommandImages
    
    
    def getListCommandsRaddose( self ):
        """
        """
        return self.m_edListCommandsRaddose
    
    
    def configure( self ):
        EDPluginExecProcessScript.configure( self )
        EDVerbose.DEBUG( "*** EDPluginRaddosev10.configure")
        self.setRequireCCP4( True )
    
    
    def preProcess( self, _edObject = None ):
        """
        """
        EDPluginExecProcessScript.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginRaddosev10.preProcess")
        self.initializeCommandExecution()
        
        
    def initializeCommandExecution( self ):
        """
        """
        self.m_edStringCommandBeamSize = self.sizeToCommand( self.getDataInput().getBeamSize(), "BEAM")
        self.addListCommandExecution (self.m_edStringCommandBeamSize )
        self.m_edStringCommandBeamFlux = self.singleValueToCommand( self.getDataInput().getBeamFlux(), "PHOSEC")
        self.addListCommandExecution( self.m_edStringCommandBeamFlux )
        self.m_edStringCommandBeamWavelength = self.singleValueToCommand( self.getDataInput().getBeamWavelength(), "WAVELENGTH")
        self.addListCommandExecution( self.m_edStringCommandBeamWavelength )
        
        self.m_edStringCommandCrystalSize = self.sizeToCommand( self.getDataInput().getCrystalSize(), "CRYSTAL")
        self.addListCommandExecution( self.m_edStringCommandCrystalSize )
        self.m_edStringCommandCrystalNRES = self.singleValueToCommand( self.getDataInput().getCrystalNRES(), "NRES")
        self.addListCommandExecution( self.m_edStringCommandCrystalNRES )
        self.m_edStringCommandCrystalNMON = self.singleValueToCommand( self.getDataInput().getCrystalNMON(), "NMON")
        self.addListCommandExecution( self.m_edStringCommandCrystalNMON )
        self.m_edStringCommandCrystalNDNA = self.singleValueToCommand( self.getDataInput().getCrystalNDNA(), "NDNA")
        self.addListCommandExecution( self.m_edStringCommandCrystalNDNA )
        self.m_edStringCommandCrystalNRNA = self.singleValueToCommand( self.getDataInput().getCrystalNRNA(), "NRNA")
        self.addListCommandExecution( self.m_edStringCommandCrystalNRNA )
        self.m_edStringCommandCrystalPATM = self.atomCompositionToCommand( self.getDataInput().getCrystalPATM(), "PATM")
        self.addListCommandExecution( self.m_edStringCommandCrystalPATM )
        self.m_edStringCommandCrystalSATM = self.atomCompositionToCommand( self.getDataInput().getCrystalSATM(), "SATM")
        self.addListCommandExecution( self.m_edStringCommandCrystalSATM )
        self.m_edStringCommandCrystalCell = self.cellToCommand( self.getDataInput().getCrystalCell(), "CELL")
        self.addListCommandExecution( self.m_edStringCommandCrystalCell )
        
        self.m_edStringCommandExposureTime = self.singleValueToCommand( self.getDataInput().getBeamExposureTime(), "EXPOSURE")
        self.addListCommandExecution( self.m_edStringCommandExposureTime )
        self.m_edStringCommandImages = self.singleValueToCommand( self.getDataInput().getNumberOfImages(), "IMAGES")
        self.addListCommandExecution( self.m_edStringCommandImages )
        
        self.addListCommandExecution( EDString("END") )
        
            
    
    def postProcess( self, _edObject = None ):
        """
        """
        EDPluginExecProcessScript.postProcess( self )
        
        edStringRaddoseError = self.readProcessErrorLogFile()
        if( (edStringRaddoseError is not None) and ( edStringRaddoseError != EDString.empty() ) ):
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginRaddosev10.postProcess', 'EDPluginRaddosev10', edStringRaddoseError )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage 
        
        resultDict = self.analyseScriptLogFileName( EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.getScriptLogFileName() ) )

        edStringSolvent = self.m_edDictResults.getValue( EDPluginRaddosev10.m_edStringSOLVENT )
        
        # ugly workaround while waiting for RADDOSE XML output file
        edStringAbsorbedDose = None
        for edStringAbsorbedDoseKeyword in EDPluginRaddosev10.m_edListABSORBED_DOSE:
            edStringAbsorbedDose = self.m_edDictResults.getValue( edStringAbsorbedDoseKeyword )
            if (edStringAbsorbedDose is not None):
                break
        
        if( edStringSolvent is None ):
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginRaddosev10.postProcess', "Raddose", "No Result for Keyword [" + EDPluginRaddosev10.m_edStringSOLVENT + "] see: " + self.getScriptLogFileName() )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage 
        
        if( edStringAbsorbedDose is None ):
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginRaddosev10.postProcess', "Raddose", "No Result for Keyword [" + edStringAbsorbedDoseKeyword + "] see: " + self.getScriptLogFileName() )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage 
        
        
        self.m_fSolvent = float( edStringSolvent )
        if( self.m_fSolvent < EDPluginRaddosev10.m_iMIN_SOLVENT_PERCENTAGE or self.m_fSolvent > EDPluginRaddosev10.m_iMAX_SOLVENT_PERCENTAGE ):
            warningMessage = EDMessage.WARNING_RADDOSE_CONSISTENCY_PROBLEM_02 % ('EDPluginRaddosev10.postProcess', "Inconsistent solvent percentage value: " + edStringSolvent )
            EDVerbose.warning( warningMessage )
            self.addWarningMessage( warningMessage )
        
        iNumberOfImages = self.getDataInput().getNumberOfImages().getValue()
        fExposureTimePerImageInSec = self.getDataInput().getBeamExposureTime().getValue()
        fTotalExposureTime = iNumberOfImages * fExposureTimePerImageInSec
        fAbsorbedDoseSpeed = float( edStringAbsorbedDose ) / fTotalExposureTime
        xsDataAbsorbedDoseRate = XSDataAbsorbedDoseRate( fAbsorbedDoseSpeed )
        
        xsDataAbsorbedDose = XSDataFloat( float( edStringAbsorbedDose ))
        
        self.m_fTimeToReachHendersonLimit = EDPluginRaddosev10.m_fHENDERSON_LIMIT / fAbsorbedDoseSpeed
        
        xsDataRaddosev10Output = XSDataRaddoseOutput()
        xsDataRaddosev10Output.setAbsorbedDose( xsDataAbsorbedDose )
        xsDataRaddosev10Output.setAbsorbedDoseRate( xsDataAbsorbedDoseRate )

        self.setDataOutput( xsDataRaddosev10Output )
        
        
    def analyseScriptLogFileName( self, _edStringFileName ):
        """
        """
        resultDict = None
        
        f = None
        try:
            f = open( _edStringFileName )
        except:
            errorMessage = EDMessage.ERROR_CANNOT_READ_FILE_02 % ('EDPluginRaddosev10.analyseScriptLogFileName', self.getScriptLogFileName() )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )    
            raise RuntimeError, errorMessage     
        
        for line in f.xreadlines():
            edStringLine = EDString( line ).replace('\n', "")
            resultDict = self.addResultToDict( edStringLine )   
            
        return resultDict
            
                    
    def addResultToDict( self, _edStringLine ):
        """
        """
        for key in self.m_edDictResults.keys():
            if ( _edStringLine.contains( key )) :
                tokenList = _edStringLine.split(" ")
                value = EDString( tokenList[-1] )
                self.m_edDictResults.setValue( EDString( key ), value )
                
        return self.m_edDictResults
                
        
    def checkParameters( self ):
        """
        Checks the data input object
        """
        
        EDPluginExecProcessScript.checkParameters( self )
        # Checks the mandatory parameters:

        self.checkMandatoryParameters( self.getDataInput(), "Data Input is None")
        self.checkMandatoryParameters( self.getDataInput().getCrystalCell(), "crystalCell")
        self.checkMandatoryParameters( self.getDataInput().getBeamWavelength(), "beamWavelength")
        self.checkMandatoryParameters( self.getDataInput().getBeamFlux(), "beamFlux")
        self.checkMandatoryParameters( self.getDataInput().getBeamSize(), "beamSize")
        
        #self.checkMandatoryParameters( self.getDataInput().getCrystalSize(), "crystalSize")
        # Default is 0.1 0.1 0.1

        # Default is 0.1 0.1
        # self.checkMandatoryParameters( self.getDataInput().getBeamExposureTime(), "beamExposureTime")
        # self.checkMandatoryParameters( self.getDataInput().getNumberOfImages(), "numberOfImages")

        self.checkImportantParameters( self.getDataInput().getCrystalNMON(), "NMON (Check your Crystal chemical composition consistency)" )
        
        
    def singleValueToCommand( self, _xsSimpleObject, _strKeyword ):
        """
        """
        command = None   
        if( _xsSimpleObject is not None ):
            command = _strKeyword
            strValue = EDString( _xsSimpleObject.getValue() )
            command = EDString( command + " " + strValue )
            
        return command
    
    
    def atomCompositionToCommand( self, _xsAtomComposition, _strKeyword ):
        """
        """
        command = None 
        if( _xsAtomComposition is not None ):
            command = _strKeyword
            atomList = _xsAtomComposition.getAtom()
            
            for atom in atomList:
                strAtomSymbol = EDString( atom.getSymbol().getValue() )
                strAtomAmount = None
                if( atom.getNumberOf() is not None ):
                    strAtomAmount = EDString( atom.getNumberOf().getValue() )
                elif( atom.getConcentration() is not None ):
                    strAtomAmount = EDString( atom.getConcentration().getValue() )
                command = EDString( command + " " + strAtomSymbol + " " + strAtomAmount )
            
        return command
    
    
    def sizeToCommand( self, _xsSize, _strKeyword ):
        """
        """
        command = None 
        if( _xsSize is not None ):
            command = _strKeyword
            strX = EDString( _xsSize.getX().getValue() )
            strY = EDString( _xsSize.getY().getValue() )
            
            strZ = None
            # if it is a 3D object
            if( _xsSize.getZ() is not None ):
                strZ = EDString( _xsSize.getZ().getValue() )
            
            command = EDString( command + " " + strX + " " + strY )
            
            if( strZ is not None ):
                command = EDString( command + " " + strZ )
            
        return command
    
    
    def cellToCommand( self, _xsCell, _strKeyword ):
        """
        """
        command = None 
        if( _xsCell is not None ):
            command = _strKeyword
            strA = EDString( _xsCell.getLength_a().getValue() )
            strB = EDString( _xsCell.getLength_b().getValue() )
            strC = EDString( _xsCell.getLength_c().getValue() )
            strAlpha = EDString( _xsCell.getAngle_alpha().getValue() )
            strBeta = EDString( _xsCell.getAngle_beta().getValue() )
            strGamma = EDString( _xsCell.getAngle_gamma().getValue() )

            command = EDString( command + " " + strA + " " + strB + " " + strC + " " + strAlpha + " " + strBeta + " " + strGamma )
            
        return command
    
    
    def generateExecutiveSummary( self, _edPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginRaddosev10.generateExecutiveSummary")
        if ( self.getStringVersion() is not None ):
            self.addExecutiveSummaryLine( self.getStringVersion() )
        xsDataRaddoseInput  = self.getDataInput()
        xsDataRaddoseOutput = self.getDataOutput()
        self.addExecutiveSummaryLine( "Execution of radiation dose estimation successful." )
        self.addExecutiveSummaryLine( "" )
        self.addExecutiveSummaryLine( "Flux                            : %21.2g [photons/s]" % xsDataRaddoseInput.getBeamFlux().getValue() )
        if( xsDataRaddoseInput.getCrystalSize() is not None ):
            self.addExecutiveSummaryLine( "Crystal size                    : %.3f x %.3f x %.3f [mm]" % ( xsDataRaddoseInput.getCrystalSize().getX().getValue(), \
                                                                                              xsDataRaddoseInput.getCrystalSize().getY().getValue(), \
                                                                                              xsDataRaddoseInput.getCrystalSize().getZ().getValue() ))
        else:
            self.addExecutiveSummaryLine( "Crystal size                    : 0.100 x 0.100 x 0.100 [mm] (not set, taking default)" )
        self.addExecutiveSummaryLine( "Beam size                       :         %.3f x %.3f [mm]" % ( xsDataRaddoseInput.getBeamSize().getX().getValue(), \
                                                                                   xsDataRaddoseInput.getBeamSize().getY().getValue() ))
        self.addExecutiveSummaryLine( "Solvent                         : %21.2f [%%]" % self.m_fSolvent )
        self.addExecutiveSummaryLine( "Dose rate                       : %21.2g [Grays/s]" % xsDataRaddoseOutput.getAbsorbedDoseRate().getValue() )
        self.addExecutiveSummaryLine( "Time to reach Henderson limit   : %21.2g [s]" % self.m_fTimeToReachHendersonLimit )

    