#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlIndexingParallelv10.py 701 2009-05-13 09:14:46Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDList
from EDImportLib import EDActionCluster

from EDPluginControl import EDPluginControl
from EDMessage       import EDMessage
from EDUtilsFile     import EDUtilsFile
from EDConfiguration import EDConfiguration

from XSDataCommon import XSPluginItem
from XSDataCommon import XSDataImage

from XSDataMXv1 import XSDataCollection
from XSDataMXv1 import XSDataCrystal
from XSDataMXv1 import XSDataSubWedge
from XSDataMXv1 import XSDataExperimentalCondition
from XSDataMXv1 import XSDataIndexingInput
from XSDataMXv1 import XSDataGeneratePredictionInput
from XSDataMXv1 import XSDataIndexingSolutionSelected

from EDPluginControlIndexingv10 import EDPluginControlIndexingv10

class EDPluginControlIndexingParallelv10( EDPluginControlIndexingv10 ):
    """
    The plugin which controls the indexing execution plugins.
    This plugin accepts several sub-wedges with one or several images each.
    The indexing is performed on all images together.
    One image with prediction is generated for each input image.
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControlIndexingv10.__init__( self )
               
        self.m_edListOfPluginIndexingName = None
        self.m_edListOfPluginIndexing = None
        self.m_edStringPluginIndexingExecutiveSummaryName = "Parallel"
        self.m_edStringPluginIndexingName = None
        self.m_edActionClusterIndexing = None


    def preProcess( self, _edObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPluginControlIndexingv10.preProcess( self, _edObject )
        self.verboseDebug( "*** EDPluginControlIndexingParallelv10.preProcess..." )
        self.m_edListOfPluginIndexingName = EDList()
        self.m_edListOfPluginIndexing = EDList()
        self.m_edListOfPluginIndexingName.add( EDString( "EDPluginControlIndexingWrapperMOSFLMv10" ) )
        self.m_edListOfPluginIndexingName.add( EDString( "EDPluginControlIndexingWrapperLabelitv10" ) )
        self.m_edStringPluginIndexingExecutiveSummaryName = "Parallel"
        self.m_edActionClusterIndexing = EDActionCluster()
        # Value to be read from the configuration...
        self.m_edActionClusterIndexing.setClusterSize(2)
        # Load the indexing plugins
        iIndex = 1
        for edStringPluginIndexingName in self.m_edListOfPluginIndexingName:
            edStringPluginIndexingBaseName = edStringPluginIndexingName + "-%02d" % iIndex
            edPluginIndexing = self.loadPlugin( edStringPluginIndexingName, edStringPluginIndexingBaseName )
            if ( edPluginIndexing is not None ):
                edPluginIndexing.setDataInput( self.getDataInput() )
                self.m_edListOfPluginIndexing.add( edPluginIndexing )
        
          
    def process( self, _edObject = None ):
        """
        """
        EDPluginControlIndexingv10.process( self, _edObject )
        self.verboseDebug( "*** EDPluginControlIndexingParallelv10.process" )
        for edPluginIndexing in self.m_edListOfPluginIndexing:
            edPluginIndexing.connectSUCCESS( self.doSuccessActionIndexingParallel )
            edPluginIndexing.connectFAILURE( self.doFailureActionIndexingParallel )                  
            self.m_edActionClusterIndexing.addAction( edPluginIndexing )
        self.m_edActionClusterIndexing.executeAction()
        self.m_edActionClusterIndexing.synchronize()
        
        
    def postProcess( self, _edObject = None ):
        EDPluginControlIndexingv10.postProcess( self, _edObject )
        self.verboseDebug( "*** EDPluginControlIndexingParallelv10.postProcess" )
        self.setDataOutput( self.m_xsDataIndexingResult )  


    def doSuccessActionIndexingParallel( self, _edPlugin = None ):
        self.verboseDebug( "*** EDPluginControlIndexingParallelv10.doSuccessActionIndexing" )
        self.retrieveSuccessMessages( _edPlugin, "EDPluginControlIndexingParallelv10.doSuccessActionIndexing" )
        # Retrieve the output from the plugin
        self.m_xsDataIndexingResult = self.getDataIndexingResult( _edPlugin )    


    def getDataIndexingResult( self, _edPlugin ):
        """
        This method retrieves the indexing results from a MOSFLM indexing plugin.
        """
        self.verboseDebug( "*** EDPluginControlIndexingParallelv10.getDataIndexingResult" )
        return _edPlugin.getDataOutput()


    def doFailureActionIndexingParallel( self,  _edPlugin = None ):
        self.verboseDebug( "*** EDPluginControlIndexingParallelv10.doFailureActionIndexing" )
        EDVerbose.screen( "Execution of " + _edPlugin.getName() + "  failed." )
        EDVerbose.screen( "Please inspect the log file for further information." )
        self.retrieveFailureMessages( _edPlugin, "EDPluginControlIndexingParallelv10.doFailureActionIndexing" )
    

    def generateExecutiveSummary( self, _edPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        self.verboseDebug( "*** EDPluginControlIndexingParallelv10.generateExecutiveSummary" )
        self.addExecutiveSummaryLine( "Summary of indexing with %s :" % self.m_edStringPluginIndexingName )
        self.addErrorWarningMessagesToExecutiveSummary( "Indexing failure! Error messages: " )
        for edPluginIndexing in self.m_edListOfPluginIndexing:
            if ( edPluginIndexing is not None ):
                self.appendExecutiveSummary( edPluginIndexing, edPluginIndexing.getPluginName() + " : ")
        