#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlIntegrationv10.py 748 2009-05-29 11:38:00Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDList
from EDImportLib import EDDict
from EDImportLib import EDActionCluster

from EDPluginControl  import EDPluginControl
from EDMessage        import EDMessage
from EDUtilsFile      import EDUtilsFile
from EDMessage        import EDMessage
from EDConfiguration  import EDConfiguration

from XSDataCommon import XSPluginItem

from XSDataMXv1 import XSDataIntegrationInput
from XSDataMXv1 import XSDataIndexingSolutionSelected
from XSDataMXv1 import XSDataExperimentalCondition
from XSDataMXv1 import XSDataCollection
from XSDataMXv1 import XSDataIntegrationResult


class EDPluginControlIntegrationv10( EDPluginControl ):
    """
    The Plugin that controls the integration.
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControl.__init__( self )
        self.setXSDataInputClass( XSDataIntegrationInput )
        self.m_edStrPluginIntegrationName = "EDPluginMOSFLMIntegrationv10"
        self.m_edPluginIntegrationList = []
        self.m_dNoSubwedges = None
        self.m_edStrCONF_CONTROL_INTEGRATION_MAX_RMS = "maxRMSSpotDeviation"
        self.m_fMaxRMSSpotDeviation = None
        self.m_xsDataExperimentalConditionRefined = None
        self.m_edActionCluster = None
        self.m_xsDataIntegrationResult = None


    def checkParameters( self ):
        """
        Checks the mandatory parameters
        """
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv10.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                               "Data Input is None" )
        self.checkMandatoryParameters( self.getDataInput().getDataCollection(),           "dataCollection" )
        self.checkMandatoryParameters( self.getDataInput().getSelectedIndexingSolution(), "selectedIndexingSolution" )


    def configure( self ):
        EDPluginControl.configure( self )
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv10.configure")
        xsPluginItem = self.getConfiguration()
        if ( xsPluginItem == None ):
            EDVerbose.DEBUG( "EDPluginControlIntegrationv10.configure: No plugin item defined." )
            xsPluginItem = XSPluginItem()
        edStringMaxRMSSpotDeviation = EDConfiguration.getStringParamValue( xsPluginItem, self.m_edStrCONF_CONTROL_INTEGRATION_MAX_RMS )
        if( edStringMaxRMSSpotDeviation == None ):
            EDVerbose.DEBUG( "EDPluginControlIntegrationv10.configure: No configuration parameter found for: " + self.m_edStrCONF_CONTROL_INTEGRATION_MAX_RMS + ", no default value." )
        else:
            self.setMaxRMSSpotDeviation( EDString( edStringMaxRMSSpotDeviation ).atof() )


    def setMaxRMSSpotDeviation( self, _fMaxRMSSpotDeviation ):
        self.m_fMaxRMSSpotDeviation = _fMaxRMSSpotDeviation
        
    def getMaxRMSSpotDeviation( self ):
        return self.m_fMaxRMSSpotDeviation


    def preProcess( self, _edObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPluginControl.preProcess( self, _edObject )
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv10.preProcess..." )
        
        self.m_edActionCluster = EDActionCluster()
        # TODO: Value to be read from the configuration... see bug #95
        self.m_edActionCluster.setClusterSize(4)

        xsDataIntegrationInput = self.getDataInput()       
        xsDataSelectedIndexingSolution     = xsDataIntegrationInput.getSelectedIndexingSolution()
        self.m_xsDataExperimentalConditionRefined = xsDataIntegrationInput.getExperimentalConditionRefined()
        # To be changed (see bug #40)
        if ( self.m_xsDataExperimentalConditionRefined is None ):
            self.m_xsDataExperimentalConditionRefined = xsDataSelectedIndexingSolution.getExperimentalConditionRefined()
        xsDataCrystalRefined               = xsDataIntegrationInput.getCrystalRefined()
        xsDataCollection                   = xsDataIntegrationInput.getDataCollection()
        xsDataSubWedgeList                 = xsDataCollection.getSubWedge()
        
        self.m_edPluginIntegrationList = EDList()
        
        iIndex = 0
        for xsDataSubWedge in xsDataSubWedgeList:
  
          edPluginIntegration = self.loadPlugin( self.m_edStrPluginIntegrationName )
          
          if ( not edPluginIntegration is None ):
              iIndex += 1
              xsDataIntegrationInputSubWedge = XSDataIntegrationInput()
              xsDataIntegrationInputSubWedge.setSelectedIndexingSolution( XSDataIndexingSolutionSelected.parseString( xsDataSelectedIndexingSolution.marshal() )  )
              xsDataIntegrationInputSubWedge.setExperimentalConditionRefined( XSDataExperimentalCondition.parseString( self.m_xsDataExperimentalConditionRefined.marshal() ) )
              xsDataCollection = XSDataCollection()
              xsDataCollection.addSubWedge( xsDataSubWedge )
              xsDataIntegrationInputSubWedge.setDataCollection( xsDataCollection )
              try:
                  from EDHandlerXSDataMOSFLMv10 import EDHandlerXSDataMOSFLMv10
                  xsDataMOSFLMInputIntegration = EDHandlerXSDataMOSFLMv10.generateXSDataMOSFLMInputIntegration( xsDataIntegrationInputSubWedge )
                  edPluginIntegration.setDataInput( xsDataMOSFLMInputIntegration )
                  edPluginIntegration.setBaseName( "%s-%02d" % ( self.m_edStrPluginIntegrationName, iIndex) )
                  edPluginIntegration.connectSUCCESS( self.doSuccessActionIntegration )
                  edPluginIntegration.connectFAILURE( self.doFailureActionIntegration )                  
                  self.m_edPluginIntegrationList.add( edPluginIntegration )
              except Exception, errorMessage:
                  self.addErrorMessage( errorMessage )
                  raise Exception, errorMessage              
          else:
              errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ('EDPluginControlIntegrationv10.preProcess', self.m_edStrPluginRaddoseName )
              EDVerbose.error( errorMessage )
              self.addErrorMessage( errorMessage )              
              raise RuntimeError, errorMessage
          
          
    def process( self, _edObject = None ):
        """
        """
        EDPluginControl.process( self, _edObject )
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv10.process" )
        for pluginIntegration in self.m_edPluginIntegrationList:
            self.m_edActionCluster.addAction( pluginIntegration )
        self.m_edActionCluster.executeAction()
        self.m_edActionCluster.synchronize()
        
        
    def doSuccessActionIntegration( self,  _edPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv10.doSuccessActionIntegration" )
        if ( _edPlugin.getErrorMessages().getNumberObjects() != 0 ):
            self.doFailureActionIntegration( _edPlugin )
        else:
            #self.synchronizeOn()
            self.retrieveSuccessMessages( _edPlugin, "EDPluginControlIntegrationv10.doSuccessActionIntegration" )        
            #self.synchronizeOff()
                


    def doFailureActionIntegration( self,  _edPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv10.doFailureActionIntegration" )
        self.retrieveFailureMessages( _edPlugin, "EDPluginControlIntegrationv10.doFailureActionIntegration" )


    def postProcess( self, _edObject = None ):
        EDPluginControl.postProcess( self, _edObject )
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv10.postProcess" )
        from EDHandlerXSDataMOSFLMv10 import EDHandlerXSDataMOSFLMv10
        for pluginIntegration in self.m_edPluginIntegrationList:
            xsDataMOSFLMOutputIntegration = pluginIntegration.getDataOutput()
            if ( xsDataMOSFLMOutputIntegration is not None):
                xsDataIntegrationSubWedgeResult = None
                try:
                    xsDataIntegrationSubWedgeResult = EDHandlerXSDataMOSFLMv10.generateXSDataIntegrationSubWedgeResult( xsDataMOSFLMOutputIntegration, self.m_xsDataExperimentalConditionRefined )
                except Exception, errorMessage:
                    self.addErrorMessage( errorMessage )
                if ( xsDataIntegrationSubWedgeResult is not None ):
                    xsDataStatisticsIntegration = xsDataIntegrationSubWedgeResult.getStatistics()
                    fRMSSpotDeviation = xsDataStatisticsIntegration.getRMSSpotDeviation().getValue()
                    if ( self.m_fMaxRMSSpotDeviation is not None ):
                        if ( self.m_fMaxRMSSpotDeviation < fRMSSpotDeviation ):
                            iImageStart  = pluginIntegration.getDataInput().getImageStart().getValue()
                            iImageEnd    = pluginIntegration.getDataInput().getImageEnd().getValue()
                            errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginControlIntegrationv10.postProcess", \
                                                                               "MOSFLM Integration : RMS spot deviation (%.3f [mm]) larger than max value from configuration (%.3f [mm]) for images %d to %d"  % \
                                                                               ( fRMSSpotDeviation, self.m_fMaxRMSSpotDeviation, iImageStart, iImageEnd ) )
                            EDVerbose.error( errorMessage )
                            self.addErrorMessage( errorMessage )
                    if ( xsDataIntegrationSubWedgeResult is not None ):
                        if ( self.m_xsDataIntegrationResult is None ):
                            self.m_xsDataIntegrationResult = XSDataIntegrationResult()
                        self.m_xsDataIntegrationResult.addIntegrationSubWedgeResult( xsDataIntegrationSubWedgeResult )
        self.setDataOutput( self.m_xsDataIntegrationResult )


    def generateExecutiveSummary( self, _edPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginControlIntegrationv10.generateExecutiveSummary" )
        self.addExecutiveSummaryLine( "Summary of integration:" )
        self.addErrorWarningMessagesToExecutiveSummary()
        for edPluginIntegration in self.m_edPluginIntegrationList:
            if ( edPluginIntegration is not None ):
                if ( edPluginIntegration.getDataOutput() is not None ):
                    self.appendExecutiveSummary( edPluginIntegration, "MOSFLM : " )
