#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlStrategyv10.py 705 2009-05-13 09:52:07Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from EDImportSystem                    import PyMath

from EDImportLib                       import EDVerbose
from EDImportLib                       import EDDebug
from EDImportLib                       import EDString
from EDImportLib                       import EDDiskExplorer

from EDPluginControl                   import EDPluginControl
from EDMessage                         import EDMessage
from EDConfiguration                   import EDConfiguration
from EDUtilsSymmetry                   import EDUtilsSymmetry

from XSDataCommon                      import XSDataString
from XSDataCommon                      import XSDataFloat
from XSDataCommon                      import XSDataInteger

from XSDataMXv1                        import XSDataStrategyInput
from XSDataMXv1                        import XSDataSampleCrystalMM
from XSDataMXv1                        import XSDataAtom
from XSDataMXv1                        import XSDataAtomicComposition
from XSDataMXv1                        import XSDataChain
from XSDataMXv1                        import XSDataStructure                   
from XSDataMXv1                        import XSDataSolvent
from XSDataMXv1                        import XSDataChemicalCompositionMM



class EDPluginControlStrategyv10( EDPluginControl ):
    """
    The Plugin that controls the strategy step
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControl.__init__( self )
               
        self.m_oedStrPluginRaddoseName = "EDPluginRaddosev10"
        self.m_oedPluginRaddose = None  
        self.m_oedHandlerXSDataRaddose = None
        
        self.m_oedStrPluginBestName = "EDPluginBestv10"
        self.m_oedPluginBest = None   
        self.m_oedHandlerXSDataBest = None
        
        self.m_oedStrCONF_SYMOP_HOME = "symopHome" 
        # Default value for the location of the symop table
        self.m_oedStrSymopHome = EDDiskExplorer.normPath( "/opt/pxsoft/ccp4-6.0.2/lib/data" )
        
        self.m_xsDataSampleCopy = None
        
        # For default chemical composition
        self.m_fAverageAminoAcidVolume=135.49
        self.m_fAverageCrystalSolventContent=0.47
        self.m_fAverageSulfurContentPerAminoacid = 0.05
        self.m_fAverageSulfurConcentration=314
        
        self.setXSDataInputClass( XSDataStrategyInput )
        
        
    def setSymopHome( self, _oedStrSymopHome ):
        """
        """
        self.m_oedStrSymopHome = _oedStrSymopHome
        
        
    def getSymopHome( self ):
        """
        """
        return self.m_oedStrSymopHome
    
    
    def preProcess( self, _oedObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPluginControl.preProcess( self, _oedObject )
        EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.preProcess..." )
        self.m_oedPluginRaddose = None
        
        xsDataSampleCrystalMM = self.getDataInput().getSample()
        
        if( xsDataSampleCrystalMM is None ):
            self.m_xsDataSampleCopy =  XSDataSampleCrystalMM()

        else:
            strXmlStringDataSample = xsDataSampleCrystalMM.marshal()
            self.m_xsDataSampleCopy = XSDataSampleCrystalMM.parseString( strXmlStringDataSample )
            
        xsDataCrystal = self.getDataInput().getCrystalRefined()
        if( xsDataCrystal is not None ):
            self.m_xsDataSampleCopy.setCrystal( xsDataCrystal )

        # Raddose is enabled only if the beam flux is set
        if( self.getDataInput().getExperimentalCondition().getBeam().getFlux() is None ):
            warningMessage = EDMessage.WARNING_CANNOT_USE_PLUGIN_03 % ('EDPluginControlStrategyv10.preProcess', self.m_oedStrPluginRaddoseName, "Beam Flux not set" )
            EDVerbose.warning( warningMessage )
            self.addWarningMessage( warningMessage )
                
        else:
            self.m_oedPluginRaddose = self.loadPlugin( self.m_oedStrPluginRaddoseName )
            
            if ( self.m_oedPluginRaddose is not None ):
                EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.preProcess: " + self.m_oedStrPluginRaddoseName + " Found... setting Data Input")

                edStringFileSymop = EDDiskExplorer.mergePath( self.getSymopHome(), "symop.lib" )

                xsDataStringSpaceGroup = self.getDataInput().getDiffractionPlan().getForcedSpaceGroup()
                # Space Group has been forced
                # Prepare chemical composition calculation with the forced Space Group (Space Group Name)
                if( xsDataStringSpaceGroup is not None):
                    edStringSpaceGroup = EDString( xsDataStringSpaceGroup.getValue() )
                    EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.preProcess: Forced Space Group Found: " + edStringSpaceGroup )
                    try:
                        edStringNumOperators = EDUtilsSymmetry.getNumberOfSymmetryOperatorsFromSpaceGroupName( edStringSpaceGroup, edStringFileSymop )
                    except Exception, detail:
                        errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginControlStrategyv10.preProcess', "Problem to calculate Number of symmetry operators", detail )
                        EDVerbose.error( errorMessage )
                        self.addErrorMessage( errorMessage )              
                        raise RuntimeError, errorMessage
                # Space Group has NOT been forced
                # Prepare chemical composition calculation with the Space Group calculated by indexing (Space Group IT Number)
                else:
                    edStringSpaceGroup = EDString( self.m_xsDataSampleCopy.getCrystal().getSpaceGroup().getITNumber().getValue() )
                    EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.preProcess: Space Group IT Number Found by indexing: " + edStringSpaceGroup )
                    try:
                        edStringNumOperators = EDUtilsSymmetry.getNumberOfSymmetryOperatorsFromSpaceGroupITNumber( edStringSpaceGroup, edStringFileSymop )
                    except Exception, detail:
                        errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginControlStrategyv10.preProcess', "Problem to calculate Number of symmetry operators", detail )
                        EDVerbose.error( errorMessage )
                        self.addErrorMessage( errorMessage )              
                        raise RuntimeError, errorMessage
                    
                if( edStringNumOperators is not None ):
                    iNumOperators = edStringNumOperators.atoi()
                else:
                    errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginControlStrategyv10.preProcess', "No symmetry operators found for Space Group: ", edStringSpaceGroup )
                    EDVerbose.error( errorMessage )
                    self.addErrorMessage( errorMessage )              
                    raise RuntimeError, errorMessage
                
                xsDataChemicalComposition = self.m_xsDataSampleCopy.getChemicalComposition()

                if( xsDataChemicalComposition is None ):
                    # create a default chemical composition and assign it to the sample
                    xsDataDefaultChemicalComposition = self.getDefaultChemicalComposition( self.m_xsDataSampleCopy, iNumOperators )
                    self.m_xsDataSampleCopy.setChemicalComposition( xsDataDefaultChemicalComposition )
                else:
                    # Check for Sulfur atoms, if none, add native sulfur atoms
                    xsDataUpdatedChemicalComposition = self.updateChemicalComposition( xsDataChemicalComposition )
                    self.m_xsDataSampleCopy.setChemicalComposition( xsDataUpdatedChemicalComposition )
                    
                # create Data Input for Raddose
                from EDHandlerXSDataRaddosev10 import EDHandlerXSDataRaddosev10
                self.m_oedHandlerXSDataRaddose = EDHandlerXSDataRaddosev10()
                xsDataBeam = self.getDataInput().getExperimentalCondition().getBeam()
                
                xsDataRaddoseInput = None
                
                try:
                    xsDataRaddoseInput = self.m_oedHandlerXSDataRaddose.getXSDataRaddoseInput( xsDataBeam, self.m_xsDataSampleCopy, iNumOperators )
                
                except Exception, detail:
                    warningMessage = EDMessage.WARNING_CANNOT_USE_PLUGIN_03 % ('EDPluginControlStrategyv10.preProcess', self.m_oedStrPluginRaddoseName, self.m_oedHandlerXSDataRaddose.getClassName() + ": " + EDString( detail ) )
                    EDDebug.controlDebug( self.m_oedHandlerXSDataRaddose, True )
                    EDVerbose.warning( warningMessage )
                    self.addWarningMessage( warningMessage )
                    
                if( xsDataRaddoseInput is not None ):
                    self.m_oedPluginRaddose.setDataInput( xsDataRaddoseInput )
                    self.m_oedPluginRaddose.setBaseDirectory( self.getWorkingDirectory() )
                    self.m_oedPluginRaddose.setBaseName( self.m_oedStrPluginRaddoseName )
                    
                    # More checks?
#                    try:
#                        self.m_oedPluginRaddose.setDataInput( xsDataRaddoseInput )
#                        self.m_oedPluginRaddose.setBaseDirectory( self.getWorkingDirectory() )
#                        self.m_oedPluginRaddose.setBaseName( self.m_oedStrPluginRaddoseName )
#                        
#                    except Exception, detail:
#                        warningMessage = EDMessage.WARNING_CANNOT_USE_PLUGIN_03 % ('EDPluginControlStrategyv10.preProcess', self.m_oedStrPluginRaddoseName, EDString( detail ) )
#                        EDDebug.controlDebug( self.m_oedPluginRaddose, True )
#                        EDVerbose.warning( warningMessage )
#                        self.addWarningMessage( warningMessage )
                        
            else:
                errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ('EDPluginControlStrategyv10.preProcess', self.m_oedStrPluginRaddoseName )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )              
                raise RuntimeError, errorMessage

        self.m_oedPluginBest = self.loadPlugin( self.m_oedStrPluginBestName )
        if ( self.m_oedPluginBest is None ):
            errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ('EDPluginControlStrategyv10.preProcess', self.m_oedStrPluginBestName )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )              
            raise RuntimeError, errorMessage
        else:
            self.m_oedPluginBest.setBaseDirectory( self.getWorkingDirectory() )
            self.m_oedPluginBest.setBaseName( self.m_oedStrPluginBestName )
        
        
    def configure( self ):
        """
        """
        EDPluginControl.configure( self )
        EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.configure")
        pluginConfiguration = self.getConfiguration()
        
        if( pluginConfiguration == None):
            warningMessage = EDMessage.WARNING_NO_PLUGIN_CONFIGURATION_ITEM_FOUND_02 % ('EDPluginControlStrategyv10.configure', self.getPluginName() )
            EDVerbose.warning( warningMessage )
            self.addWarningMessage( warningMessage )
        else:
            oedStrSymopHome = EDConfiguration.getStringParamValue( pluginConfiguration, self.m_oedStrCONF_SYMOP_HOME )
            if( oedStrSymopHome == None ):
                warningMessage = EDMessage.WARNING_NO_PARAM_CONFIGURATION_ITEM_FOUND_03 % ('EDPluginControlStrategyv10.configure', self.m_oedStrCONF_SYMOP_HOME, self.getPluginName() )
                EDVerbose.warning( warningMessage )
                self.addWarningMessage( warningMessage )
            else:
                oedStrSymopHomeNorm = EDDiskExplorer.normPath( oedStrSymopHome )
                self.setSymopHome( oedStrSymopHomeNorm )
            
    
    def process( self, _oedObject = None ):
        """
        """
        EDPluginControl.process( self, _oedObject )
        EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.process..." )
        
        # In case Beam Flux has not been set, The plugin Raddose has not been created, so it could be None
        # and it won't be launched in this case
        if( self.m_oedPluginRaddose is not None ):
            self.connectProcess( self.m_oedPluginRaddose.executeSynchronous )
            self.m_oedPluginRaddose.connectSUCCESS( self.doRaddoseToBestTransition )
            self.m_oedPluginRaddose.connectFAILURE( self.doFailureActionRaddose )
        else:
            # The plugin Best should not be None as this has been checked in the preProcess method... Double check it anyway
            if( self.m_oedPluginBest is not None ):
                self.connectProcess( self.executeBest )
            
        # Launches Best anyway whether Raddose has been launched or not
        # The plugin Best should not be None as this has been checked in the preProcess method... Double check it anyway
        if( self.m_oedPluginBest is not None ):
            self.m_oedPluginBest.connectSUCCESS( self.doSuccessActionBest )
            self.m_oedPluginBest.connectFAILURE( self.doFailureActionBest )
        
        
    def postProcess( self, _oedObject = None ):
        """
        """
        EDPluginControl.postProcess( self, _oedObject )
        EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.postProcess...")

        xsDataBestOutput = self.m_oedPluginBest.getDataOutput()
        xsDataStrategyResult = None
        
        if( xsDataBestOutput is not None ):
            xsDataStrategyResult = self.m_oedHandlerXSDataBest.getXSDataStrategyResult( xsDataBestOutput, self.getDataInput().getExperimentalCondition(), self.m_xsDataSampleCopy )
            
        self.setDataOutput( xsDataStrategyResult )
            
            
    def doRaddoseToBestTransition( self, _oedPlugin ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.doRaddoseToBestTransition" )
        self.retrieveSuccessMessages( _oedPlugin, "EDPluginControlStrategyv10.doRaddoseToBestTransition" )
        
        xsDataRaddoseOutput = self.m_oedPluginRaddose.getDataOutput()
        
        # update the strategy data with the data coming from Raddose
        self.m_xsDataSampleCopy.setAbsorbedDoseRate( xsDataRaddoseOutput.getAbsorbedDoseRate() )
        
        # Call the Best Translator layer
        from EDHandlerXSDataBestv10    import EDHandlerXSDataBestv10
        self.m_oedHandlerXSDataBest = EDHandlerXSDataBestv10()
        
        xsDataBestInput = self.m_oedHandlerXSDataBest.getXSDataBestInput( self.getDataInput().getExperimentalCondition().getBeam(),
                                                                             self.m_xsDataSampleCopy,
                                                                             self.getDataInput().getExperimentalCondition().getDetector(),
                                                                             self.getDataInput().getExperimentalCondition().getGoniostat(),
                                                                             self.getDataInput().getDiffractionPlan(),
                                                                             self.getDataInput().getBestFileContentDat(),
                                                                             self.getDataInput().getBestFileContentPar(),
                                                                             self.getDataInput().getBestFileContentHKL() )

        self.m_oedPluginBest.setDataInput( xsDataBestInput )
        self.m_oedPluginBest.executeSynchronous()
        
        
    def doFailureActionRaddose( self, _oedPlugin ):
        """
        retrieve the potential warning messages
        retrieve the potential error messages
        """
        EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.doFailureActionRaddose" )
        self.retrieveFailureMessages( _oedPlugin, "EDPluginControlStrategyv10.doFailureActionRaddose" )
        warningMessage = EDMessage.WARNING_CANNOT_USE_PLUGIN_03 % ('EDPluginControlStrategyv10.doFailureActionRaddose', self.m_oedStrPluginRaddoseName, "Raddose failure" )
        EDVerbose.warning( warningMessage )
        self.addWarningMessage( warningMessage )
        self.executeBest( self )
        
        
    def executeBest( self, _oedPlugin ):
        
        # Call the Best Translator layer
        from EDHandlerXSDataBestv10    import EDHandlerXSDataBestv10
        self.m_oedHandlerXSDataBest = EDHandlerXSDataBestv10()
        
        xsDataBestInput = self.m_oedHandlerXSDataBest.getXSDataBestInput( self.getDataInput().getExperimentalCondition().getBeam(),
                                                                             self.m_xsDataSampleCopy,
                                                                             self.getDataInput().getExperimentalCondition().getDetector(),
                                                                             self.getDataInput().getExperimentalCondition().getGoniostat(),
                                                                             self.getDataInput().getDiffractionPlan(),
                                                                             self.getDataInput().getBestFileContentDat(),
                                                                             self.getDataInput().getBestFileContentPar(),
                                                                             self.getDataInput().getBestFileContentHKL() )

        self.m_oedPluginBest.setDataInput( xsDataBestInput )
        self.m_oedPluginBest.executeSynchronous()
        
        
    def doSuccessActionBest( self, _oedPlugin ):
        """
        retrieve the potential warning messages
        """
        EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.doSuccessActionBest" )
        self.retrieveSuccessMessages( _oedPlugin, "EDPluginControlStrategyv10.doSuccessActionBest" )
        
        
    def doFailureActionBest( self, _oedPlugin ):
        """
        retrieve the potential warning messages
        retrieve the potential error messages
        """
        EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.doFailureActionBest" )
        self.retrieveFailureMessages( _oedPlugin, "EDPluginControlStrategyv10.doFailureActionBest" )

        
    def getDefaultChemicalComposition( self, _xsDataSample, _inumOperators ):
        """
        """
        xsDataCell = _xsDataSample.getCrystal().getCell()
        a = xsDataCell.getLength_a().getValue()
        b = xsDataCell.getLength_b().getValue()
        c = xsDataCell.getLength_c().getValue()
        alpha = PyMath.radians( xsDataCell.getAngle_alpha().getValue() )
        beta = PyMath.radians( xsDataCell.getAngle_beta().getValue() )
        gamma = PyMath.radians( xsDataCell.getAngle_gamma().getValue() )
                   
        fUnitCellVolume = a*b*c* ( PyMath.sqrt( 1 - PyMath.cos(alpha)*PyMath.cos(alpha) - PyMath.cos(beta)*PyMath.cos(beta) - PyMath.cos(gamma)*PyMath.cos(gamma) + 2*PyMath.cos(alpha)*PyMath.cos(beta)*PyMath.cos(gamma) ) )
        fPolymerVolume = fUnitCellVolume * ( 1 - self.m_fAverageCrystalSolventContent )
        fNumberOfMonomersPerUnitCell = fPolymerVolume / self.m_fAverageAminoAcidVolume
        fNumberOfMonomersPerAsymmetricUnit = fNumberOfMonomersPerUnitCell / _inumOperators
        iNumberOfSulfurAtom = int( round( fNumberOfMonomersPerAsymmetricUnit * self.m_fAverageSulfurContentPerAminoacid ) )
                    
        xsDataAtom = XSDataAtom()
        xsDataAtom.setSymbol( XSDataString( "S" ))
        xsDataAtom.setNumberOf( XSDataFloat( iNumberOfSulfurAtom ) )
        xsDataAtomicComposition = XSDataAtomicComposition()
        xsDataAtomicComposition.addAtom( xsDataAtom )
                    
        xsDataChain = XSDataChain()
        xsDataChain.setType( XSDataString( "protein" ))
        xsDataChain.setNumberOfMonomers( XSDataFloat( round( fNumberOfMonomersPerAsymmetricUnit )))
        xsDataChain.setNumberOfCopies( XSDataFloat( 1 ) )
        xsDataChain.setHeavyAtoms( xsDataAtomicComposition )
                    
        xsDataStructure = XSDataStructure()
        xsDataStructure.addChain( xsDataChain )
        xsDataStructure.setNumberOfCopiesInAsymmetricUnit( XSDataFloat( 1 ) )
                    
        xsDataAtomSolvent = XSDataAtom()
        xsDataAtomSolvent.setSymbol( XSDataString( "S" ))
        xsDataAtomSolvent.setConcentration( XSDataFloat( self.m_fAverageSulfurConcentration ))
        xsDataAtomicCompositionSolvent = XSDataAtomicComposition()
        xsDataAtomicCompositionSolvent.addAtom( xsDataAtomSolvent )
        xsDataSolvent = XSDataSolvent()
        xsDataSolvent.setAtoms( xsDataAtomicCompositionSolvent )
                    
        xsDataChemicalCompositionMM = XSDataChemicalCompositionMM()
        xsDataChemicalCompositionMM.setSolvent( xsDataSolvent )
        xsDataChemicalCompositionMM.setStructure( xsDataStructure )
        
        return xsDataChemicalCompositionMM
    
    def updateChemicalComposition( self, _xsDataChemicalComposition ):
        """
        """
        xsDataChemicalComposition = _xsDataChemicalComposition
        
        for chain in xsDataChemicalComposition.getStructure().getChain():
            if( chain.getType().getValue() == "protein" ):
                bSulfurExists = False
                xsDataAtomicCompositionHeavyAtoms = chain.getHeavyAtoms()
                if( xsDataAtomicCompositionHeavyAtoms is None ):
                    xsDataAtomicCompositionHeavyAtoms = XSDataAtomicComposition()
                else:
                    for heavyAtom in xsDataAtomicCompositionHeavyAtoms.getAtom():
                        if( heavyAtom.getSymbol().getValue() == "S" or heavyAtom.getSymbol().getValue() == "s" ):
                            bSulfurExists = True
                
                # all protein chains should contain sulfur atom as a percentage of the number
                # of amino acids. Add them if the user did not input them.
                if( bSulfurExists == False ):
                    iNumberOfSulfurAtom = int( round( chain.getNumberOfMonomers().getValue() * 0.05 ))
                    xsDataSulfurAtom = XSDataAtom()
                    xsDataSulfurAtom.setSymbol( XSDataString( "S" ))
                    xsDataSulfurAtom.setNumberOf( XSDataFloat( iNumberOfSulfurAtom ) )
                    xsDataAtomicCompositionHeavyAtoms.addAtom( xsDataSulfurAtom )
                    chain.setHeavyAtoms( xsDataAtomicCompositionHeavyAtoms )

        return xsDataChemicalComposition
            
    
    def generateExecutiveSummary( self, _oedPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginControlStrategyv10.generateExecutiveSummary" )
        self.addExecutiveSummaryLine( "Summary of Strategy:" )
        self.addErrorWarningMessagesToExecutiveSummary()
        
        xsDataStrategyResult = self.getDataOutput()
        
        if ( self.m_oedPluginRaddose is not None ):
            if ( self.m_oedPluginRaddose.getDataOutput() is not None ):
                self.appendExecutiveSummary( self.m_oedPluginRaddose, "Raddose : " )
                self.addExecutiveSummaryLine( "" )
                self.addExecutiveSummaryLine( "Calculations assuming polymer crystal with:" )
                self.addExecutiveSummaryLine( "------------------------------------------" )
                self.addExecutiveSummaryLine( "" )
                
                xsDataStructure = xsDataStrategyResult.getCollectionPlan()[0].getCollectionStrategy().getSample().getChemicalComposition().getStructure()
                
                self.addExecutiveSummaryLine( "Number of structure copies in asymmetric unit: %.2f" % xsDataStructure.getNumberOfCopiesInAsymmetricUnit().getValue() )
                self.addExecutiveSummaryLine( "" )
                
                chains = xsDataStructure.getChain()
                self.addExecutiveSummaryLine( "Chains:" )
                for chain in chains:
                    strChainType = chain.getType().getValue()
                    self.addExecutiveSummaryLine( "    Type                           : %s" % strChainType )
                    if( strChainType == "protein" ):
                        self.addExecutiveSummaryLine( "    Number of residues             : %.2f" % chain.getNumberOfMonomers().getValue())
                    elif( strChainType == "dna" or strChainType == "rna" ):
                        self.addExecutiveSummaryLine( "    Number of nucleotides          : %.2f" % chain.getNumberOfMonomers().getValue())
                    oedStrHeavyAtomsChain = "    Number of heavy atoms          :"
                    if( chain.getHeavyAtoms() is not None ):
                        heavyAtomsChain = chain.getHeavyAtoms().getAtom()
                        for heavyAtomChain in heavyAtomsChain:
                            oedStrHeavyAtomsChain = oedStrHeavyAtomsChain + " %s=%d" % ( heavyAtomChain.getSymbol().getValue(), heavyAtomChain.getNumberOf().getValue() )
                    else:
                        oedStrHeavyAtomsChain = oedStrHeavyAtomsChain + " No heavy atoms"
                    self.addExecutiveSummaryLine( oedStrHeavyAtomsChain )
                    
                    self.addExecutiveSummaryLine( "    Number of copies in structure  : %.2f" % chain.getNumberOfCopies().getValue() )
                    self.addExecutiveSummaryLine( "" )
                
                ligands = xsDataStructure.getLigand()
                if( len(ligands) !=0 ):
                    self.addExecutiveSummaryLine( "Ligands:" )
                    
                for ligand in ligands:
                    self.addExecutiveSummaryLine( "    Number of light atoms          : %.2f" % ligand.getNumberOfLightAtoms().getValue())
                    oedStrHeavyAtomsLigand = "    Number of heavy atoms          :"
                    if( ligand.getHeavyAtoms() is not None ):
                        heavyAtomsLigand = ligand.getHeavyAtoms().getAtom()
                        for heavyAtomLigand in heavyAtomsLigand:
                            oedStrHeavyAtomsLigand = oedStrHeavyAtomsLigand + " %s=%d" % ( heavyAtomLigand.getSymbol().getValue(), heavyAtomLigand.getNumberOf().getValue() )
                    else:
                        oedStrHeavyAtomsChain = oedStrHeavyAtomsChain + " No heavy atoms"
                    self.addExecutiveSummaryLine( oedStrHeavyAtomsLigand )
                    self.addExecutiveSummaryLine( "    Number of copies in structure  : %.2f" % chain.getNumberOfCopies().getValue() )
                    self.addExecutiveSummaryLine( "" )
                    
                    
        if ( self.m_oedPluginBest is not None ):
            if ( self.m_oedPluginBest.getDataOutput() is not None ):
                self.appendExecutiveSummary( self.m_oedPluginBest, "Best : " )
                self.addExecutiveSummaryLine( "" )
                
                