#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginExecReadImageHeaderADSCv10.py 755 2009-06-05 09:20:39Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDDiskExplorer
from EDImportLib import EDDictionary

from EDPlugin     import EDPlugin
from EDPluginExec import EDPluginExec
from EDUtilsImage import EDUtilsImage
from EDMessage    import EDMessage

from XSDataCommon import XSDataFile
from XSDataCommon import XSDataWavelength
from XSDataCommon import XSDataImage
from XSDataCommon import XSDataAngle
from XSDataCommon import XSDataLength
from XSDataCommon import XSDataTime
from XSDataCommon import XSDataString
from XSDataCommon import XSDataInteger

from XSDataMXv1 import XSDataDetector
from XSDataMXv1 import XSDataBeam
from XSDataMXv1 import XSDataGoniostat
from XSDataMXv1 import XSDataExperimentalCondition
from XSDataMXv1 import XSDataSubWedge
from XSDataMXv1 import XSDataInputReadImageHeader
from XSDataMXv1 import XSDataResultReadImageHeader


class EDPluginExecReadImageHeaderADSCv10( EDPluginExec ):

    def __init__( self ):
        """
        """
        EDPluginExec.__init__( self )
        self.setXSDataInputClass( XSDataInputReadImageHeader )
        self.m_xsDataResultReadImageHeader = None


    def checkParameters( self ):
        """
        Checks the mandatory parameters.
        """
        EDVerbose.DEBUG( "*** EDPluginExecReadImageHeaderADSCv10.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                 "Data Input is None" )
        
        
    def process( self, _edObject = None ):
        """
        """
        EDPluginExec.process( self )
        EDVerbose.DEBUG( "*** EDPluginExecReadImageHeaderADSCv10.process")
        xsDataInputReadImageHeader = self.getDataInput() 
        xsDataFile = xsDataInputReadImageHeader.getImage()
        edStringPath = EDString( xsDataFile.getPath().getValue() )
        edStringAbsolutePath = EDDiskExplorer.getAbsolutePath( edStringPath )
        edDictHeader = self.readHeaderADSC( edStringPath )
        if ( edDictHeader is None ):
            errorMessage = EDMessage.ERROR_CANNOT_READ_FILE_02 % ( "EDPluginReadImageHeaderv10.process", edStringAbsolutePath )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage          
        else:
            xsDataExperimentalCondition = XSDataExperimentalCondition()
            xsDataDetector = XSDataDetector()  

            xsDataDetector.setBeamPositionX(       XSDataLength(  EDString( edDictHeader[ "BEAM_CENTER_X" ] ).atof() ) )
            xsDataDetector.setBeamPositionY(       XSDataLength(  EDString( edDictHeader[ "BEAM_CENTER_Y" ] ).atof() ) )
            xsDataDetector.setDistance(            XSDataLength(  EDString( edDictHeader[ "DISTANCE"   ] ).atof() ) )
            fPixelSize = EDString( edDictHeader[ "PIXEL_SIZE"   ] ).atof()
            xsDataDetector.setPixelSizeX(          XSDataLength(  fPixelSize ) )
            xsDataDetector.setPixelSizeY(          XSDataLength(  fPixelSize ) )
            if "TWOTHETA" in edDictHeader.keys():
                xsDataDetector.setTwoTheta(            XSDataAngle(   EDString( edDictHeader[ "TWOTHETA"   ] ).atof() ) )
            xsDataDetector.setNumberBytesInHeader( XSDataInteger( EDString( edDictHeader[ "HEADER_BYTES"   ] ).atof()  ) )
            xsDataDetector.setSerialNumber(        XSDataInteger( EDString( edDictHeader[ "DETECTOR_SN"   ] ).atof() ) )
            xsDataDetector.setNumberPixelX(        XSDataInteger( EDString( edDictHeader[ "SIZE1"   ] ).atof() ) )
            xsDataDetector.setNumberPixelY(        XSDataInteger( EDString( edDictHeader[ "SIZE2"   ] ).atof() ) )
            xsDataDetector.setBin(                 XSDataString(  EDString( edDictHeader[ "BIN" ] ) ) )
            xsDataDetector.setDataType(            XSDataString(  EDString( edDictHeader[ "TYPE" ] ) ) )
            xsDataDetector.setByteOrder(           XSDataString(  EDString( edDictHeader[ "BYTE_ORDER" ] ) ) )
            xsDataDetector.setImageSaturation(     XSDataString(  EDString( edDictHeader[ "CCD_IMAGE_SATURATION" ] ) ) )

            # Determine type of detector...
            iNoPixelsX = xsDataDetector.getNumberPixelX().getValue()
            iNoPixelsY = xsDataDetector.getNumberPixelY().getValue()
            if ( iNoPixelsX == 2304 and iNoPixelsY == 2304 ):
                xsDataDetector.setName( XSDataString( "ADSC Q4" ) )
                xsDataDetector.setType( XSDataString( "q4" ) )
            elif ( iNoPixelsX == 1152 and iNoPixelsY == 1152 ):
                xsDataDetector.setName( XSDataString( "ADSC Q4 bin 2x2" ) )
                xsDataDetector.setType( XSDataString( "q4-2x" ) )
            elif ( iNoPixelsX == 4096 and iNoPixelsY == 4096 ):
                xsDataDetector.setName( XSDataString( "ADSC Q210" ) )
                xsDataDetector.setType( XSDataString( "q210" ) )
            elif ( iNoPixelsX == 2048 and iNoPixelsY == 2048 ):
                xsDataDetector.setName( XSDataString( "ADSC Q210 bin 2x2" ) )
                xsDataDetector.setType( XSDataString( "q210-2x" ) )
            elif ( iNoPixelsX == 6144 and iNoPixelsY == 6144 ):
                xsDataDetector.setName( XSDataString( "ADSC Q315" ) )
                xsDataDetector.setType( XSDataString( "q315" ) )
            elif ( iNoPixelsX == 3072 and iNoPixelsY == 3072 ):
                xsDataDetector.setName( XSDataString( "ADSC Q315 bin 2x2" ) )
                xsDataDetector.setType( XSDataString( "q315-2x" ) )
            else:
                errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ("EDPluginReadImageHeaderv10.process", "Unknown detector type" )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
                raise RuntimeError, errorMessage

            xsDataExperimentalCondition.setDetector( xsDataDetector )

            # Beam object
            
            xsDataBeam = XSDataBeam()
            xsDataBeam.setWavelength(   XSDataWavelength( EDString( edDictHeader[ "WAVELENGTH" ] ).atof() ) )
            xsDataBeam.setExposureTime( XSDataTime(       EDString( edDictHeader[ "TIME" ] ).atof() ) )               
            xsDataExperimentalCondition.setBeam( xsDataBeam )
            
            # Goniostat object
            xsDataGoniostat = XSDataGoniostat()
            dRotationAxisStart = EDString( edDictHeader[ "OSC_START" ] ).atof()
            dOscillationWidth = EDString( edDictHeader[ "OSC_RANGE" ] ).atof()
            xsDataGoniostat.setRotationAxisStart( XSDataAngle( dRotationAxisStart ) )
            xsDataGoniostat.setRotationAxisEnd(   XSDataAngle( dRotationAxisStart + dOscillationWidth ) )
            xsDataGoniostat.setOscillationWidth(  XSDataAngle( dOscillationWidth ) )
            edStringRotationAxis = None
            if ( "AXIS" in edDictHeader.keys() ):
                edStringRotationAxis = EDString( edDictHeader[ "AXIS" ] )
            elif ( "OSC_AXIS" in edDictHeader.keys() ):
                edStringRotationAxis = EDString( edDictHeader[ "OSC_AXIS" ] )
            else:
                errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ("EDPluginReadImageHeaderv10.process", "Neither AXIS nor OSC_AXIS header item found." )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
                raise RuntimeError, errorMessage            
            xsDataGoniostat.setRotationAxis(              XSDataString( edStringRotationAxis ) )
            xsDataExperimentalCondition.setGoniostat( xsDataGoniostat )
            
            # Create the image object
            xsDataImage = XSDataImage()
            xsDataImage.setPath( XSDataString( edStringAbsolutePath ))
            xsDataImage.setDate( XSDataString( EDString( edDictHeader[ "DATE" ] ) ) )
            edStringFileName = EDDiskExplorer.getFileName( edStringPath )
            iImageNumber = EDUtilsImage.getImageNumber( edStringFileName )
            xsDataImage.setNumber( XSDataInteger( iImageNumber ) ) 
            
            xsDataSubWedge = XSDataSubWedge()
            xsDataSubWedge.setExperimentalCondition( xsDataExperimentalCondition )
            xsDataSubWedge.addImage( xsDataImage )
            
            self.m_xsDataResultReadImageHeader = XSDataResultReadImageHeader()
            self.m_xsDataResultReadImageHeader.setSubWedge( xsDataSubWedge )



    def postProcess( self, _edObject = None ):
        """
        """
        EDPlugin.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginReadImageHeaderv10.postProcess")
        if ( self.m_xsDataResultReadImageHeader is not None ):
            self.setDataOutput( self.m_xsDataResultReadImageHeader )


    def readHeaderADSC( self, _edStringImageFileName ):
        """
        This method reads the header from an ADSC image and returns a dictionary.
        Ideally this method should be a part of another (exec) plugin.
        """
        pyFile = None
        edDictionary = None
        try:
            pyFile = open( _edStringImageFileName, "r" )
        except:
            EDVerbose.warning( "**** EDPluginExecReadImageHeaderADSCv10.readHeaderADSC: couldn't open file: " + _edStringImageFileName )

        if ( pyFile != None ):
            EDVerbose.DEBUG( "EDPluginExecReadImageHeaderADSCv10.readHeaderADSC: Reading header from image " + _edStringImageFileName )
            pyFile.seek( 0, 0 )
            pyStrLine = pyFile.readline()        
            if ( EDString( pyStrLine[0] ).equals( "{" ) ):
                edDictionary = EDDictionary()
                pyStrLine = pyFile.readline()        
                while ( pyStrLine[0] != "}" ) :
                    pyListSplit = EDString( pyStrLine ).split( "=" )
                    edStringKeyword = EDString( pyListSplit[0] )
                    pyListSplitValue = EDString( pyListSplit[1] ).split( ";" )
                    edStringValue = EDString( pyListSplitValue[0] )
                    edDictionary.add( edStringKeyword, edStringValue)
                    pyStrLine = pyFile.readline()                
            pyFile.close()
            
        return edDictionary        
